/* 
 * capi.c: j3d encoder api
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */


#define J3D_INTERNALS
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "j3dlib.h"


/*
 * add a file to the filelist queue
 * ToDo: error handling
 */
int j3d_c_add_input_file(j3d_compress_ptr cinfo, char* filename) {
     j3d_filelist_ptr l = cinfo->next_file;

     DEBUGP("Adding File: %s\n", filename);     
     
     if (l != NULL) {
         while (l->next != NULL)
             l = l->next;
         l->next = (j3d_filelist_ptr) malloc (sizeof (j3d_filelist_struct));
         
         l->next->next = NULL;
         l->next->filename = filename;
     }
     else {
         l = (j3d_filelist_ptr) malloc (sizeof (j3d_filelist_struct));
         l->next = NULL;
         l->filename = filename;
         cinfo->next_file = l;
     }
     return -1;
}




float j3d_get_compress_ratio(j3d_compress_ptr cinfo) {

    float dfactor; /* downsampling factor */
#if 0
    int i;
    dfactor = 0;
    
    /* we can't just calculate the output_bytes/input_bytes ratio,
     * since we downsampling. This should not lower the compress ratio
     */
    for(i = 0; i < cinfo->color_components; i++) {
        dfactor += (1.0 / (cinfo->downsampling_factor[i] * cinfo->downsampling_factor[i]));
    }
    dfactor = dfactor / cinfo->color_components;
#else
    dfactor = 1.0;
#endif
    
    return (dfactor * cinfo->input_bytes) / cinfo->output_bytes;
}




/*
 * creates a compress struct (cinfo)
 */
j3d_compress_ptr j3d_create_compress() {
    j3d_compress_ptr cinfo;
    cinfo = (j3d_compress_ptr) malloc(sizeof(struct j3d_compress_struct));
    /* zero the whole stuff */
    memset((void *)(cinfo), 0, (size_t)sizeof(*cinfo));
    return cinfo;
}



/*
 * initialize compression modules
 */
void j3d_init_compress(j3d_compress_ptr cinfo) {
    /* the main controller will initialize all other modules needed */
    j3d_init_c_main_controller(cinfo);
}



/*
 * start compression, use parameters from cinfo
 */
void j3d_start_compress(j3d_compress_ptr cinfo) {
    cinfo->main->compress(cinfo);
}




