/* 
 * ccoefct.c: coefficient controller
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */

#define J3D_INTERNALS
#include <stdlib.h>
#include "j3dlib.h"



/* Private subobject */
typedef struct {
    struct j3d_c_coef_controller pub; /* public fields */

    /* private variables */
    j3d_dct_coef_ptr dct_coef;
} my_coef_controller;
typedef my_coef_controller * my_coef_ptr;




static void coefct_start(j3d_compress_ptr cinfo, j3d_image_sequence_ptr in_seq) {
    int coef_finished = 0;
    
    my_coef_ptr coef = (my_coef_ptr) cinfo->coef;

//    DEBUGP("starting coeficient controller\n");
    cinfo->f3ddct->transform(cinfo, in_seq, coef->dct_coef);
    cinfo->mcu->start_seq(cinfo, coef->dct_coef);
        
    cinfo->marker->write_scan_header(cinfo);
    while(!coef_finished) {
        coef_finished = cinfo->vlc->encode_goc(cinfo);

        /* Emit restart marker if needed */
        if (cinfo->restart_interval) cinfo->marker->write_restart_marker(cinfo);

        /* write output */
	cinfo->output->write(cinfo);
    }
}




void j3d_init_c_coef_controller(j3d_compress_ptr cinfo) {
    my_coef_ptr coef;
    unsigned int array_length, comp, i;
    j3d_dct_coefcube *carray;

    coef = (my_coef_ptr) malloc(sizeof(my_coef_controller));
    

    /* init dct_coef_struct buffer */
    coef->dct_coef = (j3d_dct_coef_ptr) malloc(sizeof(j3d_dct_coef_struct));
    coef->dct_coef->color_component = (j3d_dct_cubearray *) calloc(cinfo->color_components, sizeof(j3d_dct_cubearray));
    coef->dct_coef->length = (unsigned int *) calloc(cinfo->color_components, sizeof(unsigned int *));
    coef->dct_coef->width = (unsigned int *) calloc(cinfo->color_components, sizeof(unsigned int *));
    
    for(comp = 0; comp < cinfo->color_components; comp++) {
        
        array_length = ((cinfo->width / DCTSIZE) + 1) * ((cinfo->height / DCTSIZE) + 1);
        
        carray = (j3d_dct_coefcube *) calloc(array_length, sizeof(j3d_dct_coefcube));

        for(i = 0; i < array_length; i++) {
            carray[i] = (j3d_dct_coef *) calloc(DCT_MAXSIZE3, sizeof(j3d_dct_coef));
            
        }
        coef->dct_coef->color_component[comp] = carray;
    }
    
    
    /* init the objects used */
    j3d_init_forward_3ddct(cinfo);
    j3d_init_c_mcu(cinfo);
    j3d_init_c_vlc(cinfo);
    j3d_init_c_output(cinfo);
    j3d_init_c_marker(cinfo);
    
    /*
     * assign methods
     */
    coef->pub.compress_data = coefct_start;
    
    cinfo->coef = (struct j3d_c_coef_controller *) coef;
}


