/* 
 * cinput.c: input manager module
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */


#define J3D_INTERNALS
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "j3dlib.h"

#define	ReadOk(file,buffer,len)	(read(file,buffer,len) == len)


#define FILENAME_BUFFER_LEN 9   /* 00000.bmp */



/* Private subobject */
typedef struct {
    struct j3d_c_input pub; /* public fields */

    /* private variables */
    
    int next_image;
    char filename[FILENAME_BUFFER_LEN];
} my_input;
typedef my_input * my_input_ptr;


/*
 * get next file in list
 * ToDo: memory is not released here. this is ugly
 */
static char* get_next_file(j3d_compress_ptr cinfo) {
     j3d_filelist_ptr l = cinfo->next_file;

     if(l == NULL) {
         return NULL;
     } else {
         cinfo->next_file = l->next;
         DEBUGP("next file: %s\n", l->filename);
         return l->filename;
     }
}




#if 0
/*
 * loads a single image frame from input file to outbuf
 * ToDo: error handling 
 */
static int read_rgb_image(j3d_compress_ptr cinfo, j3d_rgb_image outbuf) {
    int inbytes;
    inbytes = read(cinfo->input_file, outbuf, cinfo->width * cinfo->height * cinfo->color_components);
    cinfo->input_bytes += inbytes;
    
    return inbytes == (cinfo->width * cinfo->height * cinfo->color_components);
}
#endif




/*
 * read BMP image
 */
static int read_bmp(j3d_compress_ptr cinfo, j3d_rgb_image out_img) {
    my_input_ptr input = (my_input_ptr) cinfo->input;

    int fd;
    
    unsigned char bmpfileheader[14];
    unsigned char bmpinfoheader[64];
    unsigned char c;
    char* next_file;
    
#define GET_2B(array,offset)  ((unsigned int) array[offset] + \
			       (((unsigned int) array[offset+1]) << 8))
#define GET_4B(array,offset)  ((int32) array[offset] + \
			       ((((int32) array[offset+1]) << 8)) + \
			       ((((int32) array[offset+2]) << 16)) + \
			       ((((int32) array[offset+3]) << 24)))
    int32 bfOffBits;
    int32 headerSize;
    int32 biWidth = 0;		/* initialize to avoid compiler warning */
    int32 biHeight = 0;
    unsigned int biPlanes;
    int32 biCompression;
    int32 biXPelsPerMeter,biYPelsPerMeter;
    int32 biClrUsed = 0;
//    int mapentrysize = 0;		/* 0 indicates no colormap */
    int32 bPad;
    unsigned int biBpp;

    /* get filename and open file */
    if((next_file = get_next_file(cinfo)) == NULL) return 0;
    if(!(fd = open(next_file, O_RDONLY)) > 0) return 0;

    
    /* process header */
    if(! ReadOk(fd, bmpfileheader, 14)) return 0;
    if(GET_2B(bmpfileheader,0) != 0x4D42) return 0; /* 'BM' */
    bfOffBits = (int32) GET_4B(bmpfileheader,10);
    /* We ignore the remaining fileheader fields */
    
    /* The infoheader might be 12 bytes (OS/2 1.x), 40 bytes (Windows),
     * or 64 bytes (OS/2 2.x).  Check the first 4 bytes to find out which.
     */
    if (! ReadOk(fd, bmpinfoheader, 4)) return 0;
    headerSize = (int32) GET_4B(bmpinfoheader,0);
    if (headerSize < 12 || headerSize > 64) return 0;
    if (! ReadOk(fd, bmpinfoheader+4, headerSize-4)) return 0;
    

    switch ((int) headerSize) {
    case 12:
        /* Decode OS/2 1.x header (Microsoft calls this a BITMAPCOREHEADER) */
        biWidth = (int32) GET_2B(bmpinfoheader,4);
        biHeight = (int32) GET_2B(bmpinfoheader,6);
        biPlanes = GET_2B(bmpinfoheader,8);
        biBpp = GET_2B(bmpinfoheader,10);
        
        if(biBpp != 24) return 0; /* we only support 24bpp */
        if (biPlanes != 1) return 0;
        break;
    case 40:
    case 64:
        /* Decode Windows 3.x header (Microsoft calls this a BITMAPINFOHEADER) */
        /* or OS/2 2.x header, which has additional fields that we ignore */
        biWidth = GET_4B(bmpinfoheader,4);
        biHeight = GET_4B(bmpinfoheader,8);
        biPlanes = GET_2B(bmpinfoheader,12);
        biBpp = GET_2B(bmpinfoheader,14);
        biCompression = GET_4B(bmpinfoheader,16);
        biXPelsPerMeter = GET_4B(bmpinfoheader,24);
        biYPelsPerMeter = GET_4B(bmpinfoheader,28);
        biClrUsed = GET_4B(bmpinfoheader,32);
        /* biSizeImage, biClrImportant fields are ignored */
        
        if(biBpp != 24) return 0; /* we only support 24bpp */
        if (biPlanes != 1) return 0;
        if (biCompression != 0) return 0;

//        if (biXPelsPerMeter > 0 && biYPelsPerMeter > 0) {
//            /* Set JFIF density parameters from the BMP data */
//            cinfo->X_density = (UINT16) (biXPelsPerMeter/100); /* 100 cm per meter */
//            cinfo->Y_density = (UINT16) (biYPelsPerMeter/100);
//            cinfo->density_unit = 2;	/* dots/cm */
//        }
        break;
    default:
        return 0;
    }

    /* Compute distance to bitmap data --- will adjust for colormap below */
    bPad = bfOffBits - (headerSize + 14);


    /* Skip any remaining pad bytes */
    if (bPad < 0) return 0;			/* incorrect bfOffBits value? */


    
    while (--bPad >= 0) {
        read(fd, &c, 1);
    }
    
#if 0
    /* Compute row width in file, including padding to 4-byte boundary */
    row_width = (JDIMENSION) (biWidth * 3);
    
    while ((row_width & 3) != 0) row_width++;

    /* Allocate space for inversion array, prepare for preload pass */
    source->whole_image = (*cinfo->mem->request_virt_sarray)
        ((j_common_ptr) cinfo, JPOOL_IMAGE, FALSE,
         row_width, (JDIMENSION) biHeight, (JDIMENSION) 1);
    


    /* Allocate one-row buffer for returned data */
    source->pub.buffer = (*cinfo->mem->alloc_sarray)
        ((j_common_ptr) cinfo, JPOOL_IMAGE,
         (JDIMENSION) (biWidth * 3), (JDIMENSION) 1);
    source->pub.buffer_height = 1;
#endif
    
    cinfo->color_components = 3;
    cinfo->width = (unsigned int) biWidth;
    cinfo->height = (unsigned int) biHeight;

    /* ToDo: set info: BGR_FORMAT (in cinfo) */
    
    if(! ReadOk(fd, out_img, biWidth * biHeight * 3)) return 0;
    close(fd);

    
    /* ToDo: postprocessing, image is upside down */
    
    input->next_image++;
    
    LOWDEBUGP("BMP read: %s successful\n", input->filename);
    return -1;
}




void j3d_init_c_input(j3d_compress_ptr cinfo) {
    my_input_ptr input = (my_input_ptr) malloc(sizeof(my_input));
    
 
    cinfo->input_bytes = 0;

    input->next_image = 0;
    
    
    /* init the objects used */


    /*
     * assign methods
     */
//    input->pub.read_rgb_image = read_rgb_image;
    input->pub.read_rgb_image = read_bmp;

    
    cinfo->input = (struct j3d_c_input *) input;
}

    
