/* 
 * cj3d: Simple Program to start a 3D-JPEG compression.
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "j3dlib.h"


int main(int argc, char **argv)
{
    char ch;
    int i;
    /* initialize the cinfo struct */
    j3d_compress_ptr cinfo = j3d_create_compress();

    /* set standard parameters */
    j3d_c_set_std_params(cinfo);


    /* if no outfile specified read from stdin */
    cinfo->output_file = STDOUT_FILENO;
    cinfo->sequence_length = -1;


    
   /*
    * parse arguments
    */
    while ((ch = getopt(argc, argv, "w:h:o:l:q:")) != EOF)
    {
        switch(ch)
        {
        case 'w': /* force image width */
            cinfo->width = atoi(optarg);
            break;
        case 'h': /* force image height */
            cinfo->height = atoi(optarg);
            break;
        case 'o': /* output file*/
            if(!(cinfo->output_file = open(optarg, O_WRONLY | O_CREAT, S_IRWXU)) > 0) perror("open output file");
            break;
        case 'l': /* sequence length*/
            cinfo->sequence_length = atoi(optarg);
            break;
	case 'q': /* image quality */
            j3d_set_quality(cinfo, atoi(optarg), -1);
            break;
        default:
            printf("usage: cj3d [-o output_file] [-q quality] [-l length] -w width -h height input_files...\n");
            return (-1);
        }
    }

    if((cinfo->width == 0) || (cinfo->height == 0)) {
        perror("sorry, arguments -w, -h are mandatory in this version.\n Can't start compress.\n");
        exit(-1);
    }
    
    
    /* init the compression modules */
    j3d_init_compress(cinfo);


    /* add input files to filelist queue */
    /* all non-option arguments are input files */
    for (i = optind; i < argc; i++)
        j3d_c_add_input_file(cinfo, argv[i]);

    
    /* start compression */
    j3d_start_compress(cinfo);

    
//    printf("\ncompress ratio 1:%f\n", j3d_get_compress_ratio(cinfo));

    
    return 0;
}
