/* 
 * cmainct.c: encoder main controller module
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */

#define J3D_INTERNALS
#include <stdlib.h>
#include <stdio.h>
#include "j3dlib.h"



/* Private subobject */
typedef struct {
    struct j3d_c_main_controller pub; /* public fields */

    /* private variables */
    j3d_image_sequence_ptr image_sequence;    
    
} my_mainct;
typedef my_mainct * my_mainct_ptr;



/*
 * start encoding sequence
 */
static void mainct_start(j3d_compress_ptr cinfo)
{
    my_mainct_ptr mainct = (my_mainct_ptr) cinfo->main;
    
    int retval = -1;
    int seq = 0;

    cinfo->marker->write_file_header(cinfo);
    
    while(retval) {
        DEBUGP("Encoding sequence %i\n----------------------------------------\n" , seq);
        if((retval = cinfo->prep->pre_process_data(cinfo, mainct->image_sequence))) {
            cinfo->marker->write_frame_header(cinfo);
            cinfo->coef->compress_data(cinfo, mainct->image_sequence);
        }
        
        seq++;
        DEBUGP("\n");
        
        /* exit loop if seq-length has been reached */
        if(seq == cinfo->sequence_length) retval = 0;  
    }

    cinfo->marker->write_file_trailer(cinfo);

    
    DEBUGP("last frame reached, cleaning up\n");

    /* write remaining bits */
    cinfo->output->write(cinfo);
}




/*
 * init main controller
 * this will recursively init all modules used 
 */
void j3d_init_c_main_controller(j3d_compress_ptr cinfo) {
    my_mainct_ptr mainct;
    j3d_image * seq;
    unsigned int i, j;
    mainct = (my_mainct_ptr) malloc(sizeof(my_mainct));
    

    DEBUGP("init: main controller\n");
    
    /* alloc imagesequence buffer */
    mainct->image_sequence = (j3d_image_sequence_ptr) malloc(sizeof(j3d_image_sequence_struct));
    seq = (j3d_image *) calloc(cinfo->max_depth, sizeof(j3d_image));
    for(i = 0; i < cinfo->max_depth; i++) {
        seq[i] = (j3d_image) calloc(cinfo->color_components, sizeof(j3d_color_component));
        for(j = 0; j < cinfo->color_components; j++) {
            /*
             * color component buffer does NOT depend on downsampling factor
             * (can be used as additional buffer by color conversion routines)
             * ToDo: alloc image width/height as multiple of DCTSIZE (8)
             */
            (seq[i])[j] = (j3d_color_component) calloc(cinfo->width * cinfo->height, sizeof(j3d_sample));
        }
    }
    
    mainct->image_sequence->images = seq;
    
    
    /* init the compression modules */
    j3d_init_c_prep_controller(cinfo);
    j3d_init_c_coef_controller(cinfo);
   
    /* init the objects used */


    /*
     * assign methods
     */
    mainct->pub.compress = mainct_start;
    
    cinfo->main = (struct j3d_c_main_controller *) mainct;
}

    

