/* 
 * coutput.c: output manager
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */


#define J3D_INTERNALS
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include "j3dlib.h"

/* ToDo: find a reasonable size for the bitarray.
 * it should be large enough to hold a GOC plus some marker info */
#define BITARRAY_SIZE 1024*1024*5



/* Private subobject */
typedef struct {
    struct j3d_c_output pub; /* public fields */

    /* private variables */
} my_output;
typedef my_output * my_output_ptr;




/*
 * ToDo: handling of non-8-bit boundaries
 */
static void output_start(j3d_compress_ptr cinfo) {
    unsigned int len;
    j3d_bitarray_ptr bitarray = cinfo->output->bitarray;


    /* always write full chars */
//    if(bitarray->bits_in_buffer % 8 == 0) DEBUGP("!!! write: can't write single bits !!!\n");
    len = bitarray->bits_in_buffer / 8 + (bitarray->bits_in_buffer % 8 == 0 ? 0 : 1);
    DEBUGP("writing buffer: %i bytes\n", len);
    write(cinfo->output_file, bitarray->bitarray, len);

    cinfo->output_bytes += len;
    
    bitarray->next_output_byte = bitarray->bitarray;
    bitarray->free_in_buffer = bitarray->size;
    bitarray->put_bits = 0; /* empty put buffer */
    bitarray->bits_in_buffer = 0;
}




void j3d_init_c_output(j3d_compress_ptr cinfo) {
    my_output_ptr output;
    
    output = (my_output_ptr) malloc(sizeof(my_output));

    cinfo->output_bytes = 0;
    
    
    /* init bitarray */
    output->pub.bitarray = (j3d_bitarray_ptr) malloc(sizeof(j3d_bitarray_struct));

    output->pub.bitarray->bitarray = (unsigned char *) malloc(BITARRAY_SIZE);
    output->pub.bitarray->next_output_byte = output->pub.bitarray->bitarray;
    output->pub.bitarray->size = BITARRAY_SIZE;
    output->pub.bitarray->free_in_buffer = BITARRAY_SIZE;
    output->pub.bitarray->put_bits = 0; /* empty put buffer */
    output->pub.bitarray->bits_in_buffer = 0;


    /* init the objects used */


    /*
     * assign methods
     */
    output->pub.write = output_start;

    
    cinfo->output = (struct j3d_c_output *) output;
}

    
