/* 
 * cprepct.c: preprocessor controller
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */

#define J3D_INTERNALS
#include <stdlib.h>
#include <stdio.h>
#include "j3dlib.h"


#define RGB_COLOR_COMPONENTS 3 


/* Private subobject */
typedef struct {
    struct j3d_c_prep_controller pub; /* public fields */

    /* private variables */
    j3d_rgb_image rgb_imagebuf;
//    j3d_image imagebuf;
} my_prep_controller;

typedef my_prep_controller * my_prep_ptr;




static int prepct_start(j3d_compress_ptr cinfo, j3d_image_sequence_ptr out_seq) {
    my_prep_ptr prep = (my_prep_ptr) cinfo->prep;
    int depth = cinfo->max_depth;
    int pass = 0;
    int prep_finished = 0;

    DEBUGP("starting preprocessor:\n");
    while(!prep_finished) {
//        DEBUGP("preprocessor: pass %i, depth %i\n", pass, depth);
        if(!cinfo->input->read_rgb_image(cinfo, prep->rgb_imagebuf)) return 0;
        
        cinfo->cconvert->color_convert(cinfo, prep->rgb_imagebuf, out_seq->images[pass]);
        
        pass++;
	/* ToDo: function that can change depth of sequence (i.e. set prep_finished=TRUE) */
	/* at the moment we always encode with depth=8 */
	prep_finished = !(pass < depth); 
    }

    /* set the output sequence parameters.
    4 * ToDo: width/height may change for values which are not multiples of DCTSIZE
     */
    out_seq->depth = pass;
    out_seq->width = cinfo->width;
    out_seq->height = cinfo->height;
    return -1;
}




void j3d_init_c_prep_controller(j3d_compress_ptr cinfo) {
    my_prep_ptr prep;
    prep = (my_prep_ptr) malloc(sizeof(my_prep_controller));


    
    /* alloc RGB image buffer */
//    prep->rgb_imagebuf = (j3d_rgb_image) calloc(cinfo->width * cinfo->height * cinfo->color_components, sizeof(unsigned char));
    
    prep->rgb_imagebuf = (j3d_rgb_image) calloc(cinfo->width * cinfo->height * cinfo->color_components, sizeof(j3d_sample));



    
    /* init the objects used */
    j3d_init_c_input(cinfo);
    j3d_init_c_color_converter(cinfo);

    
    /*
     * assign methods
     */
    prep->pub.pre_process_data = prepct_start;


    cinfo->prep = (struct j3d_c_prep_controller *) prep;
}
