/* 
 * dcoefct.c: coefficient controller (decoder) object
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */

#define J3D_INTERNALS
#include <stdlib.h>
#include <stdio.h>
#include "j3dlib.h"


/* Private subobject */
typedef struct {
    struct j3d_d_coef_controller pub; /* public fields */

    /* private variables */
    j3d_dct_coef_ptr dct_coef;
    
} my_coef_controller;
typedef my_coef_controller * my_coef_ptr;



/*
 * allocate private memory
 */
static void coef_malloc(j3d_decompress_ptr cinfo) {
    unsigned int array_length, comp, i;
    j3d_dct_coefcube *carray;
    my_coef_ptr coef = (my_coef_ptr) cinfo->coef;
    
    coef->dct_coef = (j3d_dct_coef_ptr) malloc(sizeof(j3d_dct_coef_struct));
    coef->dct_coef->color_component = calloc(cinfo->color_components, sizeof(j3d_dct_cubearray));
    coef->dct_coef->length = calloc(cinfo->color_components, sizeof(unsigned int *));
    coef->dct_coef->width = calloc(cinfo->color_components, sizeof(unsigned int *));
    for(comp = 0; comp < cinfo->color_components; comp++) {
        array_length = ((cinfo->width / DCTSIZE) + 1) * ((cinfo->height / DCTSIZE) + 1);
        
        carray = (j3d_dct_coefcube *) calloc(array_length, sizeof(j3d_dct_coefcube));

        for(i = 0; i < array_length; i++) {
            carray[i] = calloc(DCT_MAXSIZE3, sizeof(j3d_dct_coef));
            
        }
        coef->dct_coef->color_component[comp] = carray;
    }

}



/*
 * returns TRUE when all cubes decoded
 */
static int coef_start(j3d_decompress_ptr cinfo, j3d_image_sequence_ptr out_seq){
    int i;
    int coef_finished = 0;

    my_coef_ptr coef = (my_coef_ptr) cinfo->coef;
    int s = DCTSIZE2; /* ToDo: find out why this is necessary */

    /* alloc memory if not already done */
    if(coef->dct_coef == NULL) coef_malloc(cinfo);
    
    
    /* now we know the size of the coef's per image-component */
    for(i = 0; i < cinfo->color_components; i++) {
        coef->dct_coef->length[i] = (((cinfo->width) * (cinfo->height)) / s);
        coef->dct_coef->width[i]  = (cinfo->width / DCTSIZE) / cinfo->downsampling_factor[i];
    }
    

    /* ToDo: set this right */
    coef->dct_coef->depth = 8;
    
    out_seq->depth = 8;
    out_seq->width = cinfo->width;
    out_seq->height = cinfo->height;

    
    cinfo->mcu->start_seq(cinfo, coef->dct_coef);

    while(!coef_finished) {
        DEBUGP("decoding group of cubes\n");
        coef_finished = cinfo->vlc->decode_goc(cinfo);

        /* get restart marker if needed */
        if (cinfo->restart_interval) cinfo->marker->read_restart_marker(cinfo);        
    }
    
    cinfo->i3ddct->transform(cinfo, coef->dct_coef, out_seq);

    return -1;
}




void j3d_init_d_coef_controller(j3d_decompress_ptr cinfo) {
    my_coef_ptr coef;
    
    coef = (my_coef_ptr) malloc(sizeof(my_coef_controller));

    coef->dct_coef = NULL;    

    
    /* init the objects used */

    j3d_init_d_mcu(cinfo);
    j3d_init_d_vlc(cinfo);
    j3d_init_inverse_3ddct(cinfo);
    

    /*
     * assign methods
     */
    coef->pub.decompress = coef_start;

    
    cinfo->coef = (struct j3d_d_coef_controller *) coef;
}

    
