/* 
 * di3ddct.c: inverse 3D-DCT module
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */

#define J3D_INTERNALS
#include <stdlib.h>
#include <stdio.h>
#include "j3dlib.h"



#define CENTERJSAMPLE	128


/* Private subobject */
typedef struct {
    struct j3d_d_inverse_3ddct pub; /* public fields */

    /* private variables */
//    float multiplier[DCT_MAXSIZE3];
    
} my_i3ddct;
typedef my_i3ddct * my_i3ddct_ptr;


#include "i3ddct.c"



/*
 * transform/quantize a DCTSIZE*DCTSIZE*depth block
 *
 * input:  imagesequence, offset
 * output: coefcube
 *
 */
static void inverse_3ddct (j3d_decompress_ptr cinfo, j3d_dct_coefcube coef, j3d_image_sequence_ptr seq, unsigned int img_off, unsigned int color_comp, j3d_quant_tbl quant_tbl)
/* This version is used for floating-point iDCT implementations. */
{
    /* This routine is heavily used, so it's worth coding it tightly. */
    unsigned int depth = seq->depth;
    float workspace[DCT_MAXSIZE3]; /* work area for iDCT subroutine */

    
    /* dequantize workspace */
    {
        register int i;
        register float *workspaceptr;
        register j3d_quant_tbl qtbl_ptr;
        
        workspaceptr = workspace;
        qtbl_ptr = quant_tbl;
        for(i = DCTSIZE2 * depth; i > 0; i--) {
            *workspaceptr++ *= (float) *qtbl_ptr++;
        }
    }
    
    
    /* Perform the iDCT */
    j3d_i3ddct(coef, workspace, depth);
    
#if 0
     {
        int x,y,z, i;
        i = 0;
        for(z = 0; z < 8; z++) {
            for(y = 0; y < 8; y++) {
                for(x = 0; x < 8; x++) {
                    DEBUGP("%f\t", workspace[i++]);
                }
                DEBUGP("\n");
            }
            
            DEBUGP("\n");
        }
        DEBUGP("\n");
     }
#endif
    
    
    /* Load data into workspace, applying unsigned->signed conversion */
    { register float *workspaceptr;
    register j3d_sample *elemptr;
    register unsigned int elemd;
    register unsigned int elemr;
//    register unsigned int elem_next_row = seq->width - DCTSIZE;
    register unsigned int elem_next_row;
    /* ToDo!: This is a big ugly hack for now, but seq->width should be an array[comp]! */ 
    elem_next_row = (color_comp == 0) ? seq->width - DCTSIZE : (seq->width >> 1) - DCTSIZE;
    

    workspaceptr = workspace;
    //    mulptr = multiplier;
    
    for(elemd = 0; elemd < depth; elemd++) {
        elemptr = &(seq->images[elemd][color_comp][img_off]);
        
        for (elemr = 0; elemr < DCTSIZE; elemr++) {

	/* unroll the inner loop */
            *elemptr++ = (j3d_sample) (((int) ((*workspaceptr++) + (float) 16384.5) - 16384) + CENTERJSAMPLE);
            *elemptr++ = (j3d_sample) (((int) ((*workspaceptr++) + (float) 16384.5) - 16384) + CENTERJSAMPLE);
            *elemptr++ = (j3d_sample) (((int) ((*workspaceptr++) + (float) 16384.5) - 16384) + CENTERJSAMPLE);
            *elemptr++ = (j3d_sample) (((int) ((*workspaceptr++) + (float) 16384.5) - 16384) + CENTERJSAMPLE);
            *elemptr++ = (j3d_sample) (((int) ((*workspaceptr++) + (float) 16384.5) - 16384) + CENTERJSAMPLE);
            *elemptr++ = (j3d_sample) (((int) ((*workspaceptr++) + (float) 16384.5) - 16384) + CENTERJSAMPLE);
            *elemptr++ = (j3d_sample) (((int) ((*workspaceptr++) + (float) 16384.5) - 16384) + CENTERJSAMPLE);
            *elemptr++ = (j3d_sample) (((int) ((*workspaceptr++) + (float) 16384.5) - 16384) + CENTERJSAMPLE);
            
//            DEBUGP("ws: %f, el=%i\n", *workspaceptr, (*elemptr )- CENTERJSAMPLE);
            elemptr += elem_next_row;  /*  set ptr to next row  */
        }
    }
    
    }
}



static void i3ddct_start(j3d_decompress_ptr cinfo, j3d_dct_coef_ptr in_coef, j3d_image_sequence_ptr out_seq) {
    int img_off;
    int color_comp;
    int cube_nr;
    int width, height;
    int x, y;


    DEBUGP("starting i3DDCT transform\n");    


    /* process all blocks */
    for(color_comp = 0; color_comp < cinfo->color_components; color_comp++) {
        width = (out_seq->width / DCTSIZE) / cinfo->downsampling_factor[color_comp];
        height = (out_seq->height / DCTSIZE) / cinfo->downsampling_factor[color_comp];
        img_off = 0;
	cube_nr = 0;
    
        for(y = height; y > 0; y--) {
	    for(x = width; x > 0; x--) {

                inverse_3ddct (cinfo, in_coef->color_component[color_comp][cube_nr], out_seq, img_off, color_comp, cinfo->quant_tbl_ptrs[cinfo->quant_tbl_no[color_comp]]);
                img_off += DCTSIZE;
                cube_nr++;
            }
            img_off += (DCTSIZE - 1) * (width * DCTSIZE);
        }
//	output->length[color_comp] = cube_nr;
	DEBUGP("  component: %i, cubes: %i\n", color_comp, cube_nr);
    }

//    output->depth = in_seq->depth;


//    inverse_3ddct(cinfo, (in_coef->color_component[0])[0], out_seq, 0, 0, i3ddct->multiplier);


#if 0
#ifdef J3D_LOWDEBUG
    /*
     * show debug info about DC values of all cubes
     */
    {
        int i;
        DEBUGP("\n\nDC Coefficients:\n----------------\n\n");
        for(i = 0; i < in_coef->length[0]; i++) {
            DEBUGP("cube %i: DC = %i\n", i, in_coef->color_component[0][i][0]);
        }
        
    }    
#endif
#endif


    
#ifdef J3D_LOWDEBUG
    /*
     * show debug info about the image and the cube
     */
    {
        int x,y,z,i;
        const int cube_nr = 100;
        const int img_off = cube_nr*DCTSIZE +  DCTSIZE * (cube_nr * DCTSIZE % out_seq->width);
        
        DEBUGP("\n\nImage Data-Cube %i:\n----------------\n\n", cube_nr);
        for(z = 0; z < 8; z++) {
            for(y = 0; y < 8; y++) {
                for(x = 0; x < 8; x++) {                
                    DEBUGP("%i\t", out_seq->images[z][1][img_off + x + y * out_seq->width]);
                }
                DEBUGP("\n");
            }
            DEBUGP("\n");
        }

        DEBUGP("\n\nCoefficient Data-Cube %i:\n-------------------------\n\n", cube_nr);
        i = 0;
        for(z = 0; z < 8; z++) {
            for(y = 0; y < 8; y++) {
                for(x = 0; x < 8; x++) {
                    DEBUGP("%i\t", (in_coef->color_component[1])[cube_nr][i++]);
                }
                DEBUGP("\n");
            }
            DEBUGP("\n");
        }
    }
#endif

    
}




void j3d_init_inverse_3ddct(j3d_decompress_ptr cinfo) {
    my_i3ddct_ptr i3ddct;
    
    i3ddct = (my_i3ddct_ptr) malloc(sizeof(my_i3ddct));
    
    
    /* init the objects used */


    
    /*
     * assign methods
     */
    i3ddct->pub.transform = i3ddct_start;

    
    cinfo->i3ddct = (struct j3d_d_inverse_3ddct *) i3ddct;
}

    
