/* 
 * dinput.c: input module
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */

#define J3D_INTERNALS
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h> 
#include "j3dlib.h"


#define INPUT_BUF_SIZE  4096	/* choose an efficiently fread'able size */


/* Private subobject */
typedef struct {
    struct j3d_d_input pub; /* public fields */


    /* private variables */
//    int infile;       /* filedescriptor for input sequence */
    int8 * buffer;        /* start of buffer */
    
} my_input;
typedef my_input * my_input_ptr;


/*
 * Fill the input buffer --- called whenever buffer is emptied.
 */

static int fill_input_buffer (j3d_decompress_ptr cinfo)
{
    my_input_ptr input = (my_input_ptr) cinfo->input;
    size_t nbytes;

    nbytes = read(cinfo->input_file, input->buffer, INPUT_BUF_SIZE);

    if (nbytes <= 0) {
        /* Insert a fake EOI marker */
        input->buffer[0] = (int8) 0xFF;
        input->buffer[1] = (int8) J3D_EOI;
        nbytes = 2;
    }

    input->pub.next_input_byte = input->buffer;
    input->pub.bytes_in_buffer = nbytes;

#ifdef J3D_LOWDEBUG
{
    int i;
    /* debug hex output of all bytes read */
    for(i = 0; i < INPUT_BUF_SIZE; i++) {
        DEBUGP("%02x ", input->buffer[i]);
        if(!(i % 0xf)) DEBUGP("\n");
    }
}
#endif
  
    return -1;
}



/*
 * consume one symbol
 */
//static int32 consume(j3d_decompress_ptr cinfo) {  
//}



static void input_close(j3d_decompress_ptr cinfo) {
//    my_input_ptr input = (my_input_ptr) cinfo->input;
    close(cinfo->input_file);
}




void j3d_init_d_input(j3d_decompress_ptr cinfo) {
    my_input_ptr input = (my_input_ptr) malloc(sizeof(my_input));

    input->buffer = calloc(INPUT_BUF_SIZE, sizeof(int8));
    

    
    /* init the objects used */


    /*
     * assign methods
     */
    input->pub.fill_input_buffer = fill_input_buffer;
    input->pub.close = input_close;

    
    cinfo->input = (struct j3d_d_input *) input;

    /* init the marker reader module */
    j3d_init_d_marker(cinfo);
}

    
