/* 
 * dj3d.c: simple program to start a 3D-JPEG decompression
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h> 
#include <fcntl.h>
#include "j3dlib.h"




int main(int argc, char **argv)
{
    char ch;   
    j3d_decompress_ptr cinfo;

    
    /* initialize the cinfo struct */
    cinfo = j3d_create_decompress();
    /* set standard parameters */
    j3d_d_set_std_params(cinfo);


    /* if no infile specified read from stdin */
    cinfo->input_file = STDIN_FILENO;
    /* if no outfile specified read from stdin */
    cinfo->output_filename = "outimg/frame";
    
    /*
     * Pars arguments
     */
    while ((ch = getopt(argc, argv, "i:")) != EOF)
    {
        switch(ch)
        {
        case 'i': /* input file */
            if(!(cinfo->input_file = open(optarg, O_RDONLY)) > 0) perror("open input file");
            break;
        default:
            return (-1);
        }
    }

    /* there should be one non-option argument, the output_mask */
    if(optind == argc - 1)
        cinfo->output_filename = argv[optind];
    else {
        perror("usage: dj3d [-i input_file] output_mask\n");
        return -1;
    }

    
    /* init the decompression modules */
    j3d_init_decompress(cinfo);
        
    /* start decompression */
    j3d_start_decompress(cinfo);

    
    exit(0);
}



