/* 
 * dmainct.c: decoder main controller module
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */

#define J3D_INTERNALS
#include <stdlib.h>
#include <stdio.h>
#include "j3dlib.h"




/* Private subobject */
typedef struct {
    struct j3d_d_main_controller pub; /* public fields */

    /* private variables */
    j3d_image_sequence_ptr image_sequence;    
} my_mainct;
typedef my_mainct * my_mainct_ptr;



/*
 * allocate private memory
 */
static void mainct_malloc(j3d_decompress_ptr cinfo) {
    my_mainct_ptr mainct = (my_mainct_ptr) cinfo->main;
    j3d_image * seq;
    unsigned int i, j;
    
    /* alloc imagesequence buffer */
    mainct->image_sequence = (j3d_image_sequence_ptr) malloc(sizeof(j3d_image_sequence_struct));
    seq = (j3d_image *) calloc(cinfo->max_depth, sizeof(j3d_image));
    for(i = 0; i < cinfo->max_depth; i++) {
        seq[i] = (j3d_image) calloc(cinfo->color_components, sizeof(j3d_color_component));
        for(j = 0; j < cinfo->color_components; j++) {
            /*
             * color component buffer does NOT depend on downsampling factor
             * (can be used as additional buffer by color conversion routines)
             * ToDo: alloc image width/height as multiple of DCTSIZE (8)
             */
            (seq[i])[j] = (j3d_color_component) calloc(cinfo->width * cinfo->height, sizeof(j3d_sample));

            //            /* color component buffer depends on downsampling factor (different for each component) */
            //            seq[i][j] = (j3d_color_component) calloc(
            //                (unsigned int) ((cinfo->width * cinfo->height) / (cinfo->downsampling_factor[j]*cinfo->downsampling_factor[j])) + 1
            //                , sizeof(j3d_sample));            
        }
    }
    mainct->image_sequence->images = seq;
}


/*
 *
 */
static void mainct_start(j3d_decompress_ptr cinfo)
{
    my_mainct_ptr mainct = (my_mainct_ptr) cinfo->main;
    int retval;
#ifdef J3D_DEBUG    
    int seq = 0;
#endif
    /* process header markers */
    if(!cinfo->marker->read_markers(cinfo))
        ERREXIT(-1, "error reading header markers\n");
    

    
    /* now we should know anything needed to alloc the imagesequence buffer */
    mainct_malloc(cinfo);
    
    retval = -1;
    while(retval) {
        DEBUGP("\nDecoding sequence %i\n----------------------------------------\n" , seq++);

//        retval = cinfo->coef->decompress(cinfo, mainct->image_sequence);
        cinfo->coef->decompress(cinfo, mainct->image_sequence);
        cinfo->postct->post_process_data(cinfo, mainct->image_sequence);
        
        /* read markers for next frame */
        retval = cinfo->marker->read_markers(cinfo);
        switch (retval) {
        case J3D_REACHED_SOS:  /* continue loop, another scan is waiting */
            retval = -1;
            break;
        case J3D_REACHED_EOI:  /* end of datastream, exit loop */
            retval = 0;
            break;
        case J3D_ERROR:        /* error decoding markers, try continuing */
            break;
        default:
            ERREXIT(-1, "mainct->start: unexpected error\n");
            break;
        }
    }
}




void j3d_init_d_main_controller(j3d_decompress_ptr cinfo) {
    my_mainct_ptr mainct = (my_mainct_ptr) malloc(sizeof(my_mainct));
    

    DEBUGP("init: main controller\n");


    /* init the decompression modules */
    j3d_init_d_input(cinfo);
    j3d_init_d_coef_controller(cinfo);
    j3d_init_d_post_controller(cinfo);
    

    /*
     * assign methods
     */
    mainct->pub.decompress = mainct_start;
    
    cinfo->main = (struct j3d_d_main_controller *) mainct;
}

    

