/* 
 * doutput.c: output manager module
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */

#define J3D_INTERNALS
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h> 
#include <fcntl.h>
#include <sys/stat.h>
#include "j3dlib.h"


#define FILENAME_BUFFER_LEN 256


/* Private subobject */
typedef struct {
    struct j3d_d_output pub; /* public fields */

    
    /* private variables */
    
    int seq; /* sequence number of output file */
    char seq_filename[FILENAME_BUFFER_LEN];
} my_output;
typedef my_output * my_output_ptr;


static void update_seq_filename(j3d_decompress_ptr cinfo) {
    my_output_ptr output = (my_output_ptr) cinfo->output;

    /* ToDo: better handling of output filenames */
    if(output->seq > 99999) {
        sprintf(output->seq_filename, "%s_overflow.rgb", cinfo->output_filename);
    }
    else {
        sprintf(output->seq_filename, "%s%05i.rgb", cinfo->output_filename, output->seq++);
    }
    


}



/*
 *
 */
static void write_rgb_image(j3d_decompress_ptr cinfo, j3d_rgb_image in_img) {
    my_output_ptr output = (my_output_ptr) cinfo->output;
    int fd;

    update_seq_filename(cinfo);
    fd = open(output->seq_filename, O_WRONLY | O_CREAT, S_IRWXU);

    write(fd, in_img, cinfo->width * cinfo->height * cinfo->color_components);

    
    close(fd);
}




void j3d_init_d_output(j3d_decompress_ptr cinfo) {
    my_output_ptr output = (my_output_ptr) malloc(sizeof(my_output));
    
    output->seq = 0;
    /* init the objects used */


    /*
     * assign methods
     */
    output->pub.write_image = write_rgb_image;

    
    cinfo->output = (struct j3d_d_output *) output;
}

    
