/* 
 * dpostct.c: postprocessing comtroller module
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */

#define J3D_INTERNALS
#include <stdlib.h>
#include <stdio.h>
#include "j3dlib.h"



/* Private subobject */
typedef struct {
    struct j3d_d_post_controller pub; /* public fields */

    /* private variables */
    j3d_rgb_image rgb_imagebuf;
} my_postct;
typedef my_postct * my_postct_ptr;




/*
 * allocate private memory
 */
static void postct_malloc(j3d_decompress_ptr cinfo) {
    my_postct_ptr postct = (my_postct_ptr) cinfo->postct;
    
    postct->rgb_imagebuf = (j3d_rgb_image) calloc(cinfo->width * cinfo->height * RGB_COLOR_COMPONENTS, sizeof(int8));
}





#if 0
static void print_comp(j3d_color_component img, int width, int sizex, int sizey) {
    int x,y;
    
    DEBUGP("\n\nImage Data:\n----------------\n\n");
    for(y = 0; y < sizey; y++) {
        DEBUGP("%i: ", y);
        for(x = 0; x < sizex; x++) {                
            DEBUGP("%i\t", img[x + y * width]);
        }
        DEBUGP("\n");
        
    }
}
#endif



/*
 * returns TRUE when all cubes encoded
 */
static int postct_start(j3d_decompress_ptr cinfo, j3d_image_sequence_ptr in_seq) {
    my_postct_ptr postct = (my_postct_ptr) cinfo->postct;
    
    int depth = in_seq->depth;
    int pass = 0;
    int post_finished = 0;

    DEBUGP("starting postprocessor: ");

    /* alloc memory if not already done */
    if(postct->rgb_imagebuf == NULL) postct_malloc(cinfo);

    
    while(!post_finished) {
        DEBUGP(".");
//        DEBUGP("postprocessor: pass %i, depth %i\n", pass, depth);

//        if(pass == 0) print_comp(in_seq->images[pass][1], 176 >> 1, 10, 144 >> 2);
        
        cinfo->cconvert->color_deconvert(cinfo, in_seq->images[pass], postct->rgb_imagebuf);
        
//        if(pass == 0) print_comp(in_seq->images[pass][1], 176, 10, 144 >> 2);
        
        cinfo->output->write_image(cinfo, postct->rgb_imagebuf);

        pass++;
	post_finished = !(pass < depth); 
    }
    DEBUGP("done.\n");
    
    return -1;
}




void j3d_init_d_post_controller(j3d_decompress_ptr cinfo) {
    my_postct_ptr postct;
    
    postct = (my_postct_ptr) malloc(sizeof(my_postct));
    
    postct->rgb_imagebuf = NULL;
    
    
    /* init the objects used */
    j3d_init_d_color_deconverter(cinfo);
    j3d_init_d_output(cinfo);
    

    /*
     * assign methods
     */
    postct->pub.post_process_data = postct_start;

    
    cinfo->postct = (struct j3d_d_post_controller *) postct;
}

    
