/* 
 * dvlc.c: variable length decoder module
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */

/* 
 * ToDo: this code is quite a mess right now, since a lot has been taken from IJG
 */

#define J3D_INTERNALS
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "j3dlib.h"



#define HUFF_LOOKAHEAD	8	/* # of bits of lookahead */
#define BIT_BUF_SIZE  32	/* size of buffer in bits */
#define MIN_GET_BITS  (BIT_BUF_SIZE-7)


typedef struct {
    /* Basic tables: (element [0] of each array is unused) */
    int32 maxcode[18];		/* largest code of length k (-1 if none) */
    /* (maxcode[17] is a sentinel to ensure jpeg_huff_decode terminates) */
    int32 valoffset[17];		/* huffval[] offset for codes of length k */
    /* valoffset[k] = huffval[] index of 1st symbol of code length k, less
     * the smallest code of length k; so given a code of length k, the
     * corresponding symbol is huffval[code + valoffset[k]]
     */

    /* Link to public Huffman table (needed only in jpeg_huff_decode) */
    j3d_huff_tbl *pub;

    /* Lookahead tables: indexed by the next HUFF_LOOKAHEAD bits of
     * the input data stream.  If the next Huffman code is no more
     * than HUFF_LOOKAHEAD bits long, we can obtain its length and
     * the corresponding symbol directly from these tables.
     */
    int look_nbits[1<<HUFF_LOOKAHEAD]; /* # bits, or 0 if too long */
    int8 look_sym[1<<HUFF_LOOKAHEAD]; /* symbol, or unused */
} d_derived_tbl;


typedef struct {
    int last_dc_val[MAX_COLOR_COMPONENTS]; /* last DC coef */
    int32 get_buffer;           /* current bit-extraction buffer */
    int bits_left;		/* # of unused bits in it */
} vlc_current_state;



typedef struct {		/* Bitreading working state within an MCU */
    /* Current data source location */
    /* We need a copy, rather than munging the original, in case of suspension */
    int8 * next_input_byte; /* => next byte to read from source */
    unsigned int bytes_in_buffer;	/* # of bytes remaining in source buffer */
    /* Bit input buffer --- note these values are kept in register variables,
     * not in this struct, inside the inner loops.
   */
    int32 get_buffer;	        /* current bit-extraction buffer */
    int bits_left;		/* # of unused bits in it */
    /* Pointer needed by j3d_fill_bit_buffer. */
    j3d_decompress_ptr cinfo;	/* back link to decompress master record */
} bitread_working_state;





/* Private subobject */
typedef struct {
    struct j3d_d_vlc pub; /* public fields */

    /* private variables */
    d_derived_tbl * dc_derived_tbls[MAX_HUFF_TBLS];
    d_derived_tbl * ac_derived_tbls[MAX_HUFF_TBLS];

//    d_derived_tbl * dc_cur_tbl;
//    d_derived_tbl * ac_cur_tbl;
   
    vlc_current_state state;		/* Bit buffer & DC state at start of MCU */
//    unsigned int curr_component;
//    unsigned int curr_cube;

} my_vlc;
typedef my_vlc * my_vlc_ptr;





static int j3d_fill_bit_buffer (bitread_working_state * state,
                                register int32 get_buffer, register int bits_left,
                                int nbits)
/* Load up the bit buffer to a depth of at least nbits */
{
    /* Copy heavily used state fields into locals (hopefully registers) */
    register int8 * next_input_byte = state->next_input_byte;
    register unsigned int bytes_in_buffer = state->bytes_in_buffer;
    j3d_decompress_ptr cinfo = state->cinfo;

    /* Attempt to load at least MIN_GET_BITS bits into get_buffer. */
    /* (It is assumed that no request will be for more than that many bits.) */
    /* We fail to do so only if we hit a marker or are forced to suspend. */


    if (cinfo->unread_marker == 0) {	/* cannot advance past a marker */
        while (bits_left < MIN_GET_BITS) {
            register int c;

            /* Attempt to read a byte */
            if (bytes_in_buffer == 0) {
                if (! (*cinfo->input->fill_input_buffer) (cinfo))
                    return 0;
                next_input_byte = cinfo->input->next_input_byte;
                bytes_in_buffer = cinfo->input->bytes_in_buffer;
            }
            bytes_in_buffer--;
            c = *next_input_byte++;

            /* If it's 0xFF, check and discard stuffed zero byte */
            if (c == 0xFF) {
                /* Loop here to discard any padding FF's on terminating marker,
                 * so that we can save a valid unread_marker value.  NOTE: we will
                 * accept multiple FF's followed by a 0 as meaning a single FF data
                 * byte.  This data pattern is not valid according to the standard.
                 */
                do {
                    if (bytes_in_buffer == 0) {
                        if (! (*cinfo->input->fill_input_buffer) (cinfo))
                            return 0;
                        next_input_byte = cinfo->input->next_input_byte;
                        bytes_in_buffer = cinfo->input->bytes_in_buffer;
                    }
                    bytes_in_buffer--;
                    c = *next_input_byte++;
                } while (c == 0xFF);

                if (c == 0) {
                    /* Found FF/00, which represents an FF data byte */
                    c = 0xFF;
                } else {
                    /* Oops, it's actually a marker indicating end of compressed data.
                     * Save the marker code for later use.
                     * Fine point: it might appear that we should save the marker into
                     * bitread working state, not straight into permanent state.  But
                     * once we have hit a marker, we cannot need to suspend within the
                     * current MCU, because we will read no more bytes from the data
                     * source.  So it is OK to update permanent state right away.
                     */
                    cinfo->unread_marker = c;
                    /* See if we need to insert some fake zero bits. */
                    goto no_more_bytes;
                }
            }

            /* OK, load c into get_buffer */
            get_buffer = (get_buffer << 8) | c;
            bits_left += 8;
        } /* end while */
    } else {
    no_more_bytes:
        /* We get here if we've read the marker that terminates the compressed
         * data segment.  There should be enough bits in the buffer register
         * to satisfy the request; if so, no problem.
         */
//        return 0;
        if (nbits > bits_left) {
            /* Uh-oh.  Report corrupted data to user and stuff zeroes into
             * the data stream, so that we can produce some kind of image.
             * We use a nonvolatile flag to ensure that only one warning message
             * appears per data segment.
             */
            if(!cinfo->vlc->insufficient_data) {
                cinfo->vlc->insufficient_data = -1;
                DEBUGP("!!! unexpectedly hit a marker !!!\n");
            }
            
            /* Fill the buffer with zero bits */
            get_buffer <<= MIN_GET_BITS - bits_left;
            bits_left = MIN_GET_BITS;
        }
    }


    /* Unload the local registers */
    state->next_input_byte = next_input_byte;
    state->bytes_in_buffer = bytes_in_buffer;
    state->get_buffer = get_buffer;
    state->bits_left = bits_left;

    return -1;
}







/* Macros to declare and load/save bitread local variables. */
#define BITREAD_STATE_VARS  \
	register int32 get_buffer;  \
	register int bits_left;  \
	bitread_working_state br_state

#define BITREAD_LOAD_STATE(cinfop,permstate)  \
	br_state.cinfo = cinfop; \
	br_state.next_input_byte = cinfop->input->next_input_byte; \
	br_state.bytes_in_buffer = cinfop->input->bytes_in_buffer; \
	get_buffer = permstate.get_buffer; \
	bits_left = permstate.bits_left;

#define BITREAD_SAVE_STATE(cinfop,permstate)  \
	cinfop->input->next_input_byte = br_state.next_input_byte; \
	cinfop->input->bytes_in_buffer = br_state.bytes_in_buffer; \
	permstate.get_buffer = get_buffer; \
	permstate.bits_left = bits_left
/*
 * These macros provide the in-line portion of bit fetching.
 * Use CHECK_BIT_BUFFER to ensure there are N bits in get_buffer
 * before using GET_BITS, PEEK_BITS, or DROP_BITS.
 * The variables get_buffer and bits_left are assumed to be locals,
 * but the state struct might not be (jpeg_huff_decode needs this).
 *	CHECK_BIT_BUFFER(state,n,action);
 *		Ensure there are N bits in get_buffer; if suspend, take action.
 *      val = GET_BITS(n);
 *		Fetch next N bits.
 *      val = PEEK_BITS(n);
 *		Fetch next N bits without removing them from the buffer.
 *	DROP_BITS(n);
 *		Discard next N bits.
 * The value N should be a simple variable, not an expression, because it
 * is evaluated multiple times.
 */

#define CHECK_BIT_BUFFER(state,nbits,action) \
	{ if (bits_left < (nbits)) {  \
	    if (! j3d_fill_bit_buffer(&(state),get_buffer,bits_left,nbits))  \
	      { action; }  \
	    get_buffer = (state).get_buffer; bits_left = (state).bits_left; } }

#define GET_BITS(nbits) \
	(((int) (get_buffer >> (bits_left -= (nbits)))) & ((1<<(nbits))-1))

#define PEEK_BITS(nbits) \
	(((int) (get_buffer >> (bits_left -  (nbits)))) & ((1<<(nbits))-1))

#define DROP_BITS(nbits) \
	(bits_left -= (nbits))


/*
 * Figure F.12: extend sign bit.
 * On some machines, a shift and add will be faster than a table lookup.
 */

#define HUFF_EXTEND(x,s)  ((x) < extend_test[s] ? (x) + extend_offset[s] : (x))
static const int extend_test[16] =   /* entry n is 2**(n-1) */
  { 0, 0x0001, 0x0002, 0x0004, 0x0008, 0x0010, 0x0020, 0x0040, 0x0080,
    0x0100, 0x0200, 0x0400, 0x0800, 0x1000, 0x2000, 0x4000 };

static const int extend_offset[16] = /* entry n is (-1 << n) + 1 */
  { 0, ((-1)<<1) + 1, ((-1)<<2) + 1, ((-1)<<3) + 1, ((-1)<<4) + 1,
    ((-1)<<5) + 1, ((-1)<<6) + 1, ((-1)<<7) + 1, ((-1)<<8) + 1,
    ((-1)<<9) + 1, ((-1)<<10) + 1, ((-1)<<11) + 1, ((-1)<<12) + 1,
    ((-1)<<13) + 1, ((-1)<<14) + 1, ((-1)<<15) + 1 };


/*
 * Code for extracting next Huffman-coded symbol from input bit stream.
 * Again, this is time-critical and we make the main paths be macros.
 *
 * We use a lookahead table to process codes of up to HUFF_LOOKAHEAD bits
 * without looping.  Usually, more than 95% of the Huffman codes will be 8
 * or fewer bits long.  The few overlength codes are handled with a loop,
 * which need not be inline code.
 *
 * Notes about the HUFF_DECODE macro:
 * 1. Near the end of the data segment, we may fail to get enough bits
 *    for a lookahead.  In that case, we do it the hard way.
 * 2. If the lookahead table contains no entry, the next code must be
 *    more than HUFF_LOOKAHEAD bits long.
 * 3. jpeg_huff_decode returns -1 if forced to suspend.
 */

#define HUFF_DECODE(result,state,htbl,failaction,slowlabel) \
{ register int nb, look; \
  if (bits_left < HUFF_LOOKAHEAD) { \
    if (! j3d_fill_bit_buffer(&state,get_buffer,bits_left, 0)) {failaction;} \
    get_buffer = state.get_buffer; bits_left = state.bits_left; \
    if (bits_left < HUFF_LOOKAHEAD) { \
      nb = 1; goto slowlabel; \
    } \
  } \
  look = PEEK_BITS(HUFF_LOOKAHEAD); \
  if ((nb = htbl->look_nbits[look]) != 0) { \
    DROP_BITS(nb); \
    result = htbl->look_sym[look]; \
  } else { \
    nb = HUFF_LOOKAHEAD+1; \
slowlabel: \
    if ((result=j3d_huff_decode(&state,get_buffer,bits_left,htbl,nb)) < 0) \
	{ failaction; } \
    get_buffer = state.get_buffer; bits_left = state.bits_left; \
  } \
}

/*
 * Out-of-line code for Huffman code decoding.
 * See jdhuff.h for info about usage.
 */

static int j3d_huff_decode (bitread_working_state * state,
                             register int32 get_buffer, register int bits_left,
                             d_derived_tbl * htbl, int min_bits)
{
  register int l = min_bits;
  register int32 code;

  /* HUFF_DECODE has determined that the code is at least min_bits */
  /* bits long, so fetch that many bits in one swoop. */

  CHECK_BIT_BUFFER(*state, l, return -1);
  code = GET_BITS(l);

  /* Collect the rest of the Huffman code one bit at a time. */
  /* This is per Figure F.16 in the JPEG spec. */

  while (code > htbl->maxcode[l]) {
    code <<= 1;
    CHECK_BIT_BUFFER(*state, 1, return -1);
    code |= GET_BITS(1);
    l++;
  }

  /* Unload the local registers */
  state->get_buffer = get_buffer;
  state->bits_left = bits_left;

  /* With garbage input we may reach the sentinel value l = 17. */

  if (l > 16) {
//    WARNMS(state->cinfo, JWRN_HUFF_BAD_CODE);
      DEBUGP("!!! bad huffman code\n");
      return 0;			/* fake a zero as the safest result */

  }

  return htbl->pub->huffval[ (int) (code + htbl->valoffset[l]) ];
}


/*
 * Decode and return one MCU's worth of Huffman-compressed coefficients.
 * The coefficients are reordered from zigzag order into natural array order,
 * but are not dequantized.
 *
 * The i'th block of the MCU is stored into the block pointed to by
 * MCU_data[i].  WE ASSUME THIS AREA HAS BEEN ZEROED BY THE CALLER.
 * (Wholesale zeroing is usually a little faster than retail...)
 *
 * Returns FALSE if data source requested suspension.  In that case no
 * changes have been made to permanent state.  (Exception: some output
 * coefficients may already have been assigned.  This is harmless for
 * this module, since we'll just re-assign them on the next call.)
 */
static int decode_cube (j3d_decompress_ptr cinfo, j3d_dct_coefcube cube_data, unsigned int depth, d_derived_tbl *dctbl, d_derived_tbl *actbl, int last_dc_val, unsigned int * coef_order_tbl) {
    my_vlc_ptr vlc = (my_vlc_ptr) cinfo->vlc;
    BITREAD_STATE_VARS;
    
//    d_derived_tbl * dctbl = vlc->dc_cur_tbl;
//    d_derived_tbl * actbl = vlc->ac_cur_tbl;
    register int s, k, r;


/* If we've run out of data, just leave the MCU set to zeroes.
 * This way, we return uniform gray for the remainder of the segment.
 */
    if (! vlc->pub.insufficient_data) {

/* Load up working state */
        BITREAD_LOAD_STATE(cinfo,vlc->state);
    


/* Decode a single block's worth of coefficients */

/* Section F.2.2.1: decode the DC coefficient difference */
        HUFF_DECODE(s, br_state, dctbl, return 0, label1);
        if (s) {
            CHECK_BIT_BUFFER(br_state, s, return 0);
            r = GET_BITS(s);
            s = HUFF_EXTEND(r, s);
        }


/* Convert DC difference to actual value, update last_dc_val */
        s += last_dc_val;

/* Output the DC coefficient (assumes coef_order_tbl[0] = 0) */
        cube_data[0] = (j3d_dct_coef) s;


/* Section F.2.2.2: decode the AC coefficients */
/* Since zeroes are skipped, output area must be cleared beforehand */
        for (k = 1; k < DCTSIZE2*depth; k++) {
            HUFF_DECODE(s, br_state, actbl, return 0, label2);
      
            r = s >> 4;
            s &= 15;
      
            if (s) {
                k += r;
                CHECK_BIT_BUFFER(br_state, s, return 0);
                r = GET_BITS(s);
                s = HUFF_EXTEND(r, s);
/* Output coefficient in natural (dezigzagged) order.
 * Note: the extra entries in jpeg_natural_order[] will save us
 * if k >= DCTSIZE2, which could happen if the data is corrupted.
 */
//            if(k > DCTSIZE2 * 8)  DEBUGP("@");
            
                cube_data[coef_order_tbl[k]] = (j3d_dct_coef) s;
            } else {
                if (r != 15)
                    break;
                k += 15;
            }
        }


//    }

/* Completed Cube, so update state */

        BITREAD_SAVE_STATE(cinfo,vlc->state);
    }

/* Account for restart interval (no-op if not using restarts) */
//  vlc->restarts_to_go--;

    return -1;
}





/*
 * Compute the derived values for a Huffman table.
 * This routine also performs some validation checks on the table.
 *
 * ToDo: Error management
 */

static void j3d_make_d_derived_tbl (j3d_decompress_ptr cinfo, int isDC, int tblno, d_derived_tbl ** pdtbl)
{
    j3d_huff_tbl *htbl;
    d_derived_tbl *dtbl;
    int p, i, l, si, numsymbols;
    int lookbits, ctr;
    char huffsize[257];
    unsigned int huffcode[257];
    unsigned int code;

    /* Note that huffsize[] and huffcode[] are filled in code-length order,
     * paralleling the order of the symbols themselves in htbl->huffval[].
     */

    /* Find the input Huffman table */
//  if (tblno < 0 || tblno >= MAX_HUFF_TBLS)
//    ERREXIT1(cinfo, JERR_NO_HUFF_TABLE, tblno);
    if (isDC) htbl = cinfo->dc_huff_tbl_ptrs[cinfo->dc_huff_tbl_no[tblno]];
    else      htbl = cinfo->ac_huff_tbl_ptrs[cinfo->ac_huff_tbl_no[tblno]];
    
//  if (htbl == NULL)
//    ERREXIT1(cinfo, JERR_NO_HUFF_TABLE, tblno);

    /* Allocate a workspace if we haven't already done so. */
    if (pdtbl[tblno] == NULL)
        pdtbl[tblno] = (d_derived_tbl *) malloc(sizeof(d_derived_tbl));

    dtbl = pdtbl[tblno];
//        dtbl = *pdtbl;
    dtbl->pub = htbl;		/* fill in back link */
 
    /* Figure C.1: make table of Huffman code length for each symbol */

    p = 0;
    for (l = 1; l <= 16; l++) {
        i = (int) htbl->bits[l];
//    if (i < 0 || p + i > 256)	/* protect against table overrun */
//      ERREXIT(cinfo, JERR_BAD_HUFF_TABLE);
        while (i--)
            huffsize[p++] = (char) l;
    }
    huffsize[p] = 0;
    numsymbols = p;
  
    /* Figure C.2: generate the codes themselves */
    /* We also validate that the counts represent a legal Huffman code tree. */
  
    code = 0;
    si = huffsize[0];
    p = 0;
    while (huffsize[p]) {
        while (((int) huffsize[p]) == si) {
            huffcode[p++] = code;
            code++;
        }
        /* code is now 1 more than the last code used for codelength si; but
         * it must still fit in si bits, since no code is allowed to be all ones.
         */
//    if (((INT32) code) >= (((INT32) 1) << si))
//      ERREXIT(cinfo, JERR_BAD_HUFF_TABLE);
        code <<= 1;
        si++;
    }

    /* Figure F.15: generate decoding tables for bit-sequential decoding */

    p = 0;
    for (l = 1; l <= 16; l++) {
        if (htbl->bits[l]) {
            /* valoffset[l] = huffval[] index of 1st symbol of code length l,
             * minus the minimum code of length l
             */
            dtbl->valoffset[l] = (int32) p - (int32) huffcode[p];
            p += htbl->bits[l];
            dtbl->maxcode[l] = huffcode[p-1]; /* maximum code of length l */
        } else {
            dtbl->maxcode[l] = -1;	/* -1 if no codes of this length */
        }
    }
    dtbl->maxcode[17] = 0xFFFFFL; /* ensures jpeg_huff_decode terminates */

    /* Compute lookahead tables to speed up decoding.
     * First we set all the table entries to 0, indicating "too long";
     * then we iterate through the Huffman codes that are short enough and
     * fill in all the entries that correspond to bit sequences starting
     * with that code.
     */

    memset((void *)(dtbl->look_nbits), 0, (size_t)sizeof(dtbl->look_nbits));

  
    p = 0;
    for (l = 1; l <= HUFF_LOOKAHEAD; l++) {
        for (i = 1; i <= (int) htbl->bits[l]; i++, p++) {
            /* l = current code's length, p = its index in huffcode[] & huffval[]. */
            /* Generate left-justified code followed by all possible bit sequences */
            lookbits = huffcode[p] << (HUFF_LOOKAHEAD-l);
            for (ctr = 1 << (HUFF_LOOKAHEAD-l); ctr > 0; ctr--) {
                dtbl->look_nbits[lookbits] = l;
                dtbl->look_sym[lookbits] = htbl->huffval[p];
                lookbits++;
            }
        }
    }

    /* Validate symbols as being reasonable.
     * For AC tables, we make no check, but accept all byte values 0..255.
     * For DC tables, we require the symbols to be in range 0..15.
     * (Tighter bounds could be applied depending on the data depth and mode,
     * but this is sufficient to ensure safe decoding.)
     */
    if (isDC) {
        for (i = 0; i < numsymbols; i++) {
//      int sym = htbl->huffval[i];
//      if (sym < 0 || sym > 15)
//	ERREXIT(cinfo, JERR_BAD_HUFF_TABLE);
        }
    }
}



/*
 * decode one MCU
 * returns TRUE if last MCU in sequence decoded
 */
static int decode_mcu(j3d_decompress_ptr cinfo) {
    my_vlc_ptr vlc = (my_vlc_ptr) cinfo->vlc;
    j3d_dct_coefcube cube;
    d_derived_tbl *dctbl;
    d_derived_tbl *actbl;
    unsigned int curr_comp;
    unsigned int depth;
    int retval = 0;

    
    while(!(retval & J3D_MCU_COMPLETE) && ((retval = cinfo->mcu->get_next_cube(cinfo, &cube, &curr_comp, &depth)) & J3D_MCU_OK)) {


        dctbl = vlc->dc_derived_tbls[cinfo->dc_huff_tbl_no[curr_comp]];
        actbl = vlc->ac_derived_tbls[cinfo->ac_huff_tbl_no[curr_comp]];

        if(actbl == NULL) {
            j3d_make_d_derived_tbl(cinfo,  0, cinfo->ac_huff_tbl_no[curr_comp], vlc->ac_derived_tbls);
            actbl = vlc->ac_derived_tbls[cinfo->ac_huff_tbl_no[curr_comp]];
        }
        if(dctbl == NULL) {
            j3d_make_d_derived_tbl(cinfo, -1, cinfo->dc_huff_tbl_no[curr_comp], vlc->dc_derived_tbls);
            dctbl = vlc->dc_derived_tbls[cinfo->dc_huff_tbl_no[curr_comp]];
        }

        /* decode cube */
        memset(cube, 0, (size_t) (DCT_MAXSIZE3 * sizeof(j3d_dct_coef))); /* decode_cube needs zeroed coefs */
        if(! decode_cube(cinfo, cube, depth, dctbl, actbl, vlc->state.last_dc_val[curr_comp], cinfo->coef_zigzag_order_8)) {
            DEBUGP("!decode_cube: hit marker 0x%02x!\n", cinfo->unread_marker); /* happens on marker hit */
        }
        
        /* save DC value */
        vlc->state.last_dc_val[curr_comp] = cube[0];
    }
    
    return (retval & J3D_MCU_SEQ_COMPLETE) || (retval & J3D_MCU_NO_MORE_BLOCKS);
}


    
/*
 * returns TRUE when all cubes are decoded
 */
static int vlc_start(j3d_decompress_ptr cinfo){
    my_vlc_ptr vlc = (my_vlc_ptr) cinfo->vlc;
    int interval_remain;
    int retval = 0;
    
    

    vlc->pub.insufficient_data = 0;

    /* init state */

    vlc->state.last_dc_val[0] = 0;
    vlc->state.last_dc_val[1] = 0;
    vlc->state.last_dc_val[2] = 0;
    
    vlc->state.bits_left -= vlc->state.bits_left % 8; /* interval will start on next byte */
    

    /* process interval */
    interval_remain = cinfo->restart_interval;
    while(interval_remain > 0) { 
        if(decode_mcu(cinfo)) { /* end of sequence (no more MCU's) */
            interval_remain = 0;
            retval = -1;
        }
        else interval_remain--;
    }
    
    
    return retval;
}



void j3d_init_d_vlc(j3d_decompress_ptr cinfo) {
    my_vlc_ptr vlc;
    int i;
    
    vlc = (my_vlc_ptr) malloc(sizeof(my_vlc));


    for(i = 0; i < MAX_HUFF_TBLS; i++) {
        vlc->dc_derived_tbls[i] = NULL;
        vlc->ac_derived_tbls[i] = NULL;
    }
    
        

    /* init the objects used */


    /*
     * assign methods
     */
    vlc->pub.decode_goc = vlc_start;

    
    cinfo->vlc = (struct j3d_d_vlc *) vlc;
}



