/* 
 * f3ddct.c: forward 3D-DCT core
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */


#define C1 0.509795579
#define C2 0.541196102
#define C3 0.601344887
#define C4 0.707106796
#define C5 0.899976280
#define C6 1.306562974
#define C7 2.562916516


/*
 * Perform the forward DCT on one cube of samples.
 *
 * ToDo: implement all possible cube depths 
 */
static void j3d_f3ddct(float *data, int depth) {
    float tmp0, tmp1, tmp2, tmp3, tmp4, tmp5, tmp6, tmp7;
    float tmp00, tmp01, tmp02, tmp03, tmp04, tmp05, tmp06, tmp07;
    int ctr, ctr2;
    float *dataptr;
    
    
    /* Pass 1: process rows (x-axis) */

    dataptr = data;
    for (ctr = DCTSIZE * depth - 1; ctr >= 0; ctr--) {
        tmp0 =  dataptr[0] + dataptr[7];
        tmp1 = (dataptr[0] - dataptr[7]) * C1;
        tmp2 =  dataptr[1] + dataptr[6];
        tmp3 = (dataptr[1] - dataptr[6]) * C3;
        tmp4 =  dataptr[3] + dataptr[4];
        tmp5 = (dataptr[3] - dataptr[4]) * C7;
        tmp6 =  dataptr[2] + dataptr[5];
        tmp7 = (dataptr[2] - dataptr[5]) * C5;
    
        tmp00 =  tmp0 + tmp4;
        tmp01 = (tmp0 - tmp4) * C2;
        tmp02 =  tmp2 + tmp6;
        tmp03 = (tmp2 - tmp6) * C6;
        tmp04 =  tmp1 + tmp5;
        tmp05 = (tmp1 - tmp5) * C2;
        tmp06 =  tmp3 + tmp7;
        tmp07 = (tmp3 - tmp7) * C6;

        tmp2 =  tmp01 + tmp03;
        tmp3 = (tmp01 - tmp03) * C4;
        tmp4 =  tmp04 + tmp06;
        tmp5 = (tmp04 - tmp06) * C4;
        tmp6 =  tmp05 + tmp07;
        tmp7 = (tmp05 - tmp07) * C4;

            
        dataptr[0] = (tmp00 + tmp02) * C4;
        dataptr[4] = (tmp00 - tmp02) * C4;

        dataptr[1] = tmp4 + tmp6 + tmp7;
        dataptr[2] = tmp2 + tmp3;
        dataptr[3] = tmp5 + tmp6 + tmp7;
        dataptr[5] = tmp5 + tmp7;
        dataptr[6] = tmp3;
        dataptr[7] = tmp7;

/*      { */
/*          int x; */
/*          for(x = 0; x < 8; x++) { */
/*              printf("%f\t", dataptr[x]); */
/*          } */
/*          printf("\n"); */
/*      } */

    
    
        dataptr += DCTSIZE;		/* advance pointer to next row */
    }

    
    
    /* Pass 2: process columns. (y-axis) */

    dataptr = data;
    for(ctr2 = depth - 1; ctr2 >= 0; ctr2--) {
        for (ctr = DCTSIZE - 1; ctr >= 0; ctr--) {
            tmp0 =  dataptr[DCTSIZE*0] + dataptr[DCTSIZE*7];
            tmp1 = (dataptr[DCTSIZE*0] - dataptr[DCTSIZE*7]) * C1;
            tmp2 =  dataptr[DCTSIZE*1] + dataptr[DCTSIZE*6];
            tmp3 = (dataptr[DCTSIZE*1] - dataptr[DCTSIZE*6]) * C3;
            tmp4 =  dataptr[DCTSIZE*3] + dataptr[DCTSIZE*4];
            tmp5 = (dataptr[DCTSIZE*3] - dataptr[DCTSIZE*4]) * C7;
            tmp6 =  dataptr[DCTSIZE*2] + dataptr[DCTSIZE*5];
            tmp7 = (dataptr[DCTSIZE*2] - dataptr[DCTSIZE*5]) * C5;
    
            tmp00 =  tmp0 + tmp4;
            tmp01 = (tmp0 - tmp4) * C2;
            tmp02 =  tmp2 + tmp6;
            tmp03 = (tmp2 - tmp6) * C6;
            tmp04 =  tmp1 + tmp5;
            tmp05 = (tmp1 - tmp5) * C2;
            tmp06 =  tmp3 + tmp7;
            tmp07 = (tmp3 - tmp7) * C6;

            tmp2 =  tmp01 + tmp03;
            tmp3 = (tmp01 - tmp03) * C4;
            tmp4 =  tmp04 + tmp06;
            tmp5 = (tmp04 - tmp06) * C4;
            tmp6 =  tmp05 + tmp07;
            tmp7 = (tmp05 - tmp07) * C4;

            
            dataptr[DCTSIZE*0] = (tmp00 + tmp02) * C4;
            dataptr[DCTSIZE*4] = (tmp00 - tmp02) * C4;

            dataptr[DCTSIZE*1] = tmp4 + tmp6 + tmp7;
            dataptr[DCTSIZE*2] = tmp2 + tmp3;
            dataptr[DCTSIZE*3] = tmp5 + tmp6 + tmp7;
            dataptr[DCTSIZE*5] = tmp5 + tmp7;
            dataptr[DCTSIZE*6] = tmp3;
            dataptr[DCTSIZE*7] = tmp7;

            dataptr++;			/* advance pointer to next column */
        }
        dataptr += DCTSIZE2 - 8;  /* advance to next image in sequence */
    }

    

    /* Pass 3: process image seuence. (z-axis) */

    dataptr = data;
    if(depth == 8) {
        for (ctr = DCTSIZE2 - 1; ctr >= 0; ctr--) {
            tmp0 =  dataptr[DCTSIZE2*0] + dataptr[DCTSIZE2*7];
            tmp1 = (dataptr[DCTSIZE2*0] - dataptr[DCTSIZE2*7]) * C1;
            tmp2 =  dataptr[DCTSIZE2*1] + dataptr[DCTSIZE2*6];
            tmp3 = (dataptr[DCTSIZE2*1] - dataptr[DCTSIZE2*6]) * C3;
            tmp4 =  dataptr[DCTSIZE2*3] + dataptr[DCTSIZE2*4];
            tmp5 = (dataptr[DCTSIZE2*3] - dataptr[DCTSIZE2*4]) * C7;
            tmp6 =  dataptr[DCTSIZE2*2] + dataptr[DCTSIZE2*5];
            tmp7 = (dataptr[DCTSIZE2*2] - dataptr[DCTSIZE2*5]) * C5;
    
            tmp00 =  tmp0 + tmp4;
            tmp01 = (tmp0 - tmp4) * C2;
            tmp02 =  tmp2 + tmp6;
            tmp03 = (tmp2 - tmp6) * C6;
            tmp04 =  tmp1 + tmp5;
            tmp05 = (tmp1 - tmp5) * C2;
            tmp06 =  tmp3 + tmp7;
            tmp07 = (tmp3 - tmp7) * C6;

            tmp2 =  tmp01 + tmp03;
            tmp3 = (tmp01 - tmp03) * C4;
            tmp4 =  tmp04 + tmp06;
            tmp5 = (tmp04 - tmp06) * C4;
            tmp6 =  tmp05 + tmp07;
            tmp7 = (tmp05 - tmp07) * C4;

            
            dataptr[DCTSIZE2*0] = (tmp00 + tmp02) * C4;
            dataptr[DCTSIZE2*4] = (tmp00 - tmp02) * C4;

            dataptr[DCTSIZE2*1] = tmp4 + tmp6 + tmp7;
            dataptr[DCTSIZE2*2] = tmp2 + tmp3;
            dataptr[DCTSIZE2*3] = tmp5 + tmp6 + tmp7;
            dataptr[DCTSIZE2*5] = tmp5 + tmp7;
            dataptr[DCTSIZE2*6] = tmp3;
            dataptr[DCTSIZE2*7] = tmp7;

            dataptr++;			/* advance pointer to next column (or row) */
        }
    } else if(depth == 4) {
        for (ctr = DCTSIZE2 - 1; ctr >= 0; ctr--) {
            /* ToDo: ... */
            
            dataptr++;			/* advance pointer to next column (or row) */
        }
    }
}
