/* 
 * i3ddct.c: inverse 3D-DCT core
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */


#define C1 0.509795579
#define C2 0.541196102
#define C3 0.601344887
#define C4 0.707106796
#define C5 0.899976280
#define C6 1.306562974
#define C7 2.562916516

#define IC1 1.0 / C1
#define IC2 1.0 / C2
#define IC3 1.0 / C3
#define IC4 1.0 / C4
#define IC5 1.0 / C5
#define IC6 1.0 / C6
#define IC7 1.0 / C7

#define SCALE (256.0 * 0.707106796) / 4.0


/*
 * Perform the forward DCT on one cube of samples.
 *
 * ToDo!: - implement all possible cube depths
 *        - !optimization!
 */
static void j3d_i3ddct(j3d_dct_coefcube in_data, float *out_data, int depth) {
    float tmp;
    float tmp0, tmp1, tmp2, tmp3, tmp4, tmp5, tmp6, tmp7;
    float tmp00, tmp01, tmp02, tmp03, tmp04, tmp05, tmp06, tmp07;
    int ctr, ctr2;
    float *dataptr;
    j3d_dct_coefcube in_dataptr;
    

    /* Pass 1: process image seuence. (z-axis) */

    in_dataptr = in_data;
    dataptr = out_data;
    if(depth == 8) {
        for (ctr = DCTSIZE2 - 1; ctr >= 0; ctr--) {

            tmp2 = (float) (in_dataptr[DCTSIZE2*2] - in_dataptr[DCTSIZE2*6]);
            tmp4 = (float) (in_dataptr[DCTSIZE2*5] - in_dataptr[DCTSIZE2*7] + in_dataptr[DCTSIZE2*1] - in_dataptr[DCTSIZE2*3]);
            tmp5 = (float) (in_dataptr[DCTSIZE2*5] - in_dataptr[DCTSIZE2*7]);
            tmp6 = (float) (in_dataptr[DCTSIZE2*3] - in_dataptr[DCTSIZE2*5]);

            tmp = (float) (in_dataptr[DCTSIZE2*4] * IC4);
            tmp00 = ((in_dataptr[DCTSIZE2*0] * IC4) + tmp)/2.0;
            tmp02 = ((in_dataptr[DCTSIZE2*0] * IC4) - tmp)/2.0;
            tmp = (float) (in_dataptr[DCTSIZE2*6] * IC4);
            tmp01 = (tmp2 + tmp)/2.0;
            tmp03 = (tmp2 - tmp)/2.0;
            tmp = tmp5 * IC4;
            tmp04 = (tmp4 + tmp)/2.0;
            tmp06 = (tmp4 - tmp)/2.0;
            tmp = (float) (in_dataptr[DCTSIZE2*7] * IC4);
            tmp05 = (tmp6 + tmp)/2.0;
            tmp07 = (tmp6 - tmp)/2.0;


            tmp = tmp01 * IC2;
            tmp0 = (tmp00 + tmp)/2.0;
            tmp4 = (tmp00 - tmp)/2.0;
            tmp = tmp03 * IC6;
            tmp2 = (tmp02 + tmp)/2.0;
            tmp6 = (tmp02 - tmp)/2.0;
            tmp = tmp05 * IC2;
            tmp1 = (tmp04 + tmp)/2.0;
            tmp5 = (tmp04 - tmp)/2.0;
            tmp = tmp07 * IC6;
            tmp3 = (tmp06 + tmp)/2.0;
            tmp7 = (tmp06 - tmp)/2.0;

            tmp = tmp1 * IC1;
            dataptr[DCTSIZE2*0] = (tmp0 + tmp) / 2.0;
            dataptr[DCTSIZE2*7] = (tmp0 - tmp) / 2.0;
            tmp = tmp3 * IC3;
            dataptr[DCTSIZE2*1] = (tmp2 + tmp) / 2.0;
            dataptr[DCTSIZE2*6] = (tmp2 - tmp) / 2.0;
            tmp = tmp5 * IC7;
            dataptr[DCTSIZE2*3] = (tmp4 + tmp) / 2.0;
            dataptr[DCTSIZE2*4] = (tmp4 - tmp) / 2.0;
            tmp = tmp7 * IC5;
            dataptr[DCTSIZE2*2] = (tmp6 + tmp) / 2.0;
            dataptr[DCTSIZE2*5] = (tmp6 - tmp) / 2.0;

            dataptr++;			/* advance pointer to next column (or row) */
            in_dataptr++;
       }
    } else if(depth == 4) {
        for (ctr = DCTSIZE2 - 1; ctr >= 0; ctr--) {
            /* ToDo: ... */
            
            dataptr++;			/* advance pointer to next column (or row) */
        }
    }


 
    /* Pass 2: process columns. (y-axis) */

    dataptr = out_data;
    for(ctr2 = depth - 1; ctr2 >= 0; ctr2--) {
        for (ctr = DCTSIZE - 1; ctr >= 0; ctr--) {

            tmp2 = dataptr[DCTSIZE*2] - dataptr[DCTSIZE*6];
            tmp4 = dataptr[DCTSIZE*5] - dataptr[DCTSIZE*7] + dataptr[DCTSIZE*1] - dataptr[DCTSIZE*3];
            tmp5 = dataptr[DCTSIZE*5] - dataptr[DCTSIZE*7];
            tmp6 = dataptr[DCTSIZE*3] - dataptr[DCTSIZE*5];

            tmp = dataptr[DCTSIZE*4] * IC4;
            tmp00 = ((dataptr[DCTSIZE*0] * IC4) + tmp)/2.0;
            tmp02 = ((dataptr[DCTSIZE*0] * IC4) - tmp)/2.0;
            tmp = dataptr[DCTSIZE*6] * IC4;
            tmp01 = (tmp2 + tmp)/2.0;
            tmp03 = (tmp2 - tmp)/2.0;
            tmp = tmp5 * IC4;
            tmp04 = (tmp4 + tmp)/2.0;
            tmp06 = (tmp4 - tmp)/2.0;
            tmp = dataptr[DCTSIZE*7] * IC4;
            tmp05 = (tmp6 + tmp)/2.0;
            tmp07 = (tmp6 - tmp)/2.0;


            tmp = tmp01 * IC2;
            tmp0 = (tmp00 + tmp)/2.0;
            tmp4 = (tmp00 - tmp)/2.0;
            tmp = tmp03 * IC6;
            tmp2 = (tmp02 + tmp)/2.0;
            tmp6 = (tmp02 - tmp)/2.0;
            tmp = tmp05 * IC2;
            tmp1 = (tmp04 + tmp)/2.0;
            tmp5 = (tmp04 - tmp)/2.0;
            tmp = tmp07 * IC6;
            tmp3 = (tmp06 + tmp)/2.0;
            tmp7 = (tmp06 - tmp)/2.0;

            tmp = tmp1 * IC1;
            dataptr[DCTSIZE*0] = (tmp0 + tmp) / 2.0;
            dataptr[DCTSIZE*7] = (tmp0 - tmp) / 2.0;
            tmp = tmp3 * IC3;
            dataptr[DCTSIZE*1] = (tmp2 + tmp) / 2.0;
            dataptr[DCTSIZE*6] = (tmp2 - tmp) / 2.0;
            tmp = tmp5 * IC7;
            dataptr[DCTSIZE*3] = (tmp4 + tmp) / 2.0;
            dataptr[DCTSIZE*4] = (tmp4 - tmp) / 2.0;
            tmp = tmp7 * IC5;
            dataptr[DCTSIZE*2] = (tmp6 + tmp) / 2.0;
            dataptr[DCTSIZE*5] = (tmp6 - tmp) / 2.0;
            
            dataptr++;			/* advance pointer to next column */

        }
        dataptr += DCTSIZE2 - 8;  /* advance to next image in sequence */
    }

    
    /* Pass 3: process rows (x-axis) */


    dataptr = out_data;
   for (ctr = DCTSIZE * depth - 1; ctr >= 0; ctr--) {
        tmp2 = (dataptr[2] - dataptr[6]);
        tmp4 = (dataptr[5] - dataptr[7] + dataptr[1] - dataptr[3]);
        tmp5 = (dataptr[5] - dataptr[7]);
        tmp6 = (dataptr[3] - dataptr[5]);

        tmp = (dataptr[4] * IC4);
        tmp00 = ((dataptr[0] * IC4) + tmp)/2.0;
        tmp02 = ((dataptr[0] * IC4) - tmp)/2.0;
        tmp = (dataptr[6] * IC4);
        tmp01 = (tmp2 + tmp)/2.0;
        tmp03 = (tmp2 - tmp)/2.0;
        tmp = tmp5 * IC4;
        tmp04 = (tmp4 + tmp)/2.0;
        tmp06 = (tmp4 - tmp)/2.0;
        tmp = (dataptr[7] * IC4);
        tmp05 = (tmp6 + tmp)/2.0;
        tmp07 = (tmp6 - tmp)/2.0;


        tmp = tmp01 * IC2;
        tmp0 = (tmp00 + tmp)/2.0;
        tmp4 = (tmp00 - tmp)/2.0;
        tmp = tmp03 * IC6;
        tmp2 = (tmp02 + tmp)/2.0;
        tmp6 = (tmp02 - tmp)/2.0;
        tmp = tmp05 * IC2;
        tmp1 = (tmp04 + tmp)/2.0;
        tmp5 = (tmp04 - tmp)/2.0;
        tmp = tmp07 * IC6;
        tmp3 = (tmp06 + tmp)/2.0;
        tmp7 = (tmp06 - tmp)/2.0;

        tmp = tmp1 * IC1;
        dataptr[0] = ((tmp0 + tmp) / 2.0) * SCALE;
        dataptr[7] = ((tmp0 - tmp) / 2.0) * SCALE;
        tmp = tmp3 * IC3;
        dataptr[1] = ((tmp2 + tmp) / 2.0) * SCALE;
        dataptr[6] = ((tmp2 - tmp) / 2.0) * SCALE;
        tmp = tmp5 * IC7;
        dataptr[3] = ((tmp4 + tmp) / 2.0) * SCALE;
        dataptr[4] = ((tmp4 - tmp) / 2.0) * SCALE;
        tmp = tmp7 * IC5;
        dataptr[2] = ((tmp6 + tmp) / 2.0) * SCALE;
        dataptr[5] = ((tmp6 - tmp) / 2.0) * SCALE;

    
        dataptr += DCTSIZE;		/* advance pointer to next row */
   }


#if 0
   printf("!!!!!!!!!!!!!!!!!!!!!!!!!!!\n\n");

   {
        int x,y,z, i;
        i = 0;
        for(z = 0; z < 8; z++) {
            for(y = 0; y < 8; y++) {
                for(x = 0; x < 8; x++) {
                    printf("%f\t", out_data[i++]);
                }
                printf("\n");
            }
            printf("\n");
        }
        printf("\n");
     }
   printf("!!!!!!!!!!!!!!!!!!!!!!!!!!!\n\n");
#endif
    

}
