/* 
 * j3dint.h: internal header file
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */


/* return values of get_next_cube in cmcu.c */
#define J3D_MCU_OK               1
#define J3D_MCU_COMPLETE         2
#define J3D_MCU_SEQ_COMPLETE     4
#define J3D_MCU_NO_MORE_BLOCKS   8



/*
 * compression structs
 */



/* Compression main control */
struct j3d_c_main_controller {
    void (*compress) (j3d_compress_ptr cinfo);
};

/* Compression preprocessing control (downsampling input buffer control) */
struct j3d_c_prep_controller {
    int (*pre_process_data) (j3d_compress_ptr cinfo,
			      j3d_image_sequence_ptr out_seq);
};

/* Coefficient buffer control */
struct j3d_c_coef_controller {
    void (*compress_data) (j3d_compress_ptr cinfo,
			   j3d_image_sequence_ptr in_seq);
};

/* input manager */
struct j3d_c_input {
    int (*read_rgb_image) (j3d_compress_ptr cinfo,
                           j3d_rgb_image out_img);
};


/* Colorspace conversion and downsampling */
struct j3d_c_color_converter {
    void (*color_convert) (j3d_compress_ptr cinfo,
                           j3d_rgb_image input_buf,
                           j3d_image output_buf);
};


/* forward 3D-DCT */
struct j3d_c_forward_3ddct {
    void (*transform) (j3d_compress_ptr cinfo,
		       j3d_image_sequence_ptr in_seq,
                       j3d_dct_coef_ptr output);
};

/* MCU manager */
struct j3d_c_mcu {
    void (*start_seq) (j3d_compress_ptr cinfo,
                       j3d_dct_coef_ptr in_coef);
    
    int (*get_next_cube) (j3d_compress_ptr cinfo,
                          j3d_dct_coefcube * out_cube,
                          unsigned int *out_comp,
                          unsigned int *out_depth);
};

/* variable length coder */
struct j3d_c_vlc {
    int  (*encode_goc) (j3d_compress_ptr cinfo);
};

/* output control */
struct j3d_c_output {
    void (*write) (j3d_compress_ptr cinfo);
    
    j3d_bitarray_ptr bitarray;
};

/* Marker writing */
struct j3d_c_marker {
    void (*write_file_header)  (j3d_compress_ptr cinfo);
    void (*write_frame_header) (j3d_compress_ptr cinfo);
    void (*write_scan_header)  (j3d_compress_ptr cinfo);
    void (*write_file_trailer) (j3d_compress_ptr cinfo);
    void (*write_restart_marker)  (j3d_compress_ptr cinfo);
    void (*write_tables_only)  (j3d_compress_ptr cinfo);
};



/*
 * decompression structs
 */

/* decompression main control */
struct j3d_d_main_controller {
    void (*decompress) (j3d_decompress_ptr cinfo);
};


/* Coefficient controller */
struct j3d_d_coef_controller {
    int (*decompress) (j3d_decompress_ptr cinfo,
                        j3d_image_sequence_ptr output);
    

};


/* decompression input controller */
struct j3d_d_input {
//    int   (*read_header) (j3d_decompress_ptr cinfo);
    int (*fill_input_buffer) (j3d_decompress_ptr cinfo);
    void  (*close) (j3d_decompress_ptr cinfo);


    int8 * next_input_byte;
    unsigned int bytes_in_buffer;
};

/* Marker reading */
struct j3d_d_marker {
    int (*read_markers)         (j3d_decompress_ptr cinfo);
    int (*read_restart_marker)  (j3d_decompress_ptr cinfo);

    /* State of marker reader --- nominally internal, but applications
     * supplying COM or APPn handlers might like to know the state.
     */
    int saw_SOI;		/* found SOI? */
    int saw_SOF;		/* found SOF? */
    int next_restart_num;		/* next restart number expected (0-7) */
    unsigned int discarded_bytes;	/* # of bytes skipped looking for a marker */
};


/* MCU manager */
struct j3d_d_mcu {
    void (*start_seq) (j3d_decompress_ptr cinfo,
                       j3d_dct_coef_ptr in_coef);
    
    int (*get_next_cube) (j3d_decompress_ptr cinfo,
                          j3d_dct_coefcube * out_cube,
                          unsigned int *out_comp,
                          unsigned int *out_depth);
};


/* variable length de-coder */
struct j3d_d_vlc {
    int (*decode_goc) (j3d_decompress_ptr cinfo);
    int insufficient_data;
};


/* inverse 3D-DCT */
struct j3d_d_inverse_3ddct {
    void (*transform) (j3d_decompress_ptr cinfo,
                       j3d_dct_coef_ptr in_coef,
                       j3d_image_sequence_ptr out_seq);
};

/* postprocessing controller */
struct j3d_d_post_controller {
    int (*post_process_data) (j3d_decompress_ptr cinfo,
                              j3d_image_sequence_ptr in_seq);
    

};

/* Colorspace conversion and upsampling */
struct j3d_d_color_deconverter {
    void (*color_deconvert) (j3d_decompress_ptr cinfo,
                             j3d_image input_buf,
                             j3d_rgb_image output_buf);
};

/* output manager */
struct j3d_d_output {
    void (*write_image) (j3d_decompress_ptr cinfo,
                         j3d_rgb_image in_img);
};




/*
 * global macro definitions
 */


/* error macros */
#define ERREXIT(code, format, ...)  { fprintf (stderr, format, ## __VA_ARGS__); exit(code); }



/* debug output macros */
#ifdef J3D_LOWDEBUG
#define J3D_DEBUG
#define LOWDEBUGP(format, ...) printf (format, ## __VA_ARGS__)
#else
#define LOWDEBUGP(format, ...)
#endif

#ifdef J3D_DEBUG
#define DEBUGP(format, ...) printf (format, ## __VA_ARGS__)
#else
#define DEBUGP(format, ...)
#endif


/*
 * internal exports
 */
extern void j3d_init_c_main_controller();

extern void j3d_init_c_prep_controller();
extern void j3d_init_c_input();
extern void j3d_init_c_color_converter();

extern void j3d_init_c_coef_controller();
extern void j3d_init_forward_3ddct();
extern void j3d_init_c_mcu();
extern void j3d_init_c_vlc();
extern void j3d_init_c_output();
extern void j3d_init_c_marker();



extern void j3d_init_d_main_controller();

extern void j3d_init_d_coef_controller();
extern void j3d_init_d_input();
extern void j3d_init_d_marker();
extern void j3d_init_d_mcu();
extern void j3d_init_d_vlc();
extern void j3d_init_inverse_3ddct();

extern void j3d_init_d_post_controller();
extern void j3d_init_d_color_deconverter();
extern void j3d_init_d_output();


extern unsigned int * j3d_std_coef_zigzag();
extern void j3d_add_huff_table();
