/* 
 * j3dlib.h: library header file 
 * Copyright (C) 2001 Axel Burri <aburri@ee.ethz.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * This program also contains code from the free IJG JPEG codec
 * software by Thomas G. Lane (www.ijg.org)
 *
 */


#ifndef J3DLIB_H
#define J3DLIB_H




#define DCTSIZE		    8   /* 3DDCT blocksize for x, y */
#define DCTSIZE2	    DCTSIZE*DCTSIZE

#define DCT_MAXDEPTH        8   /* max allowed 3DDCT blocksize for z */
#define DCT_MAXSIZE3	    DCTSIZE2*DCT_MAXDEPTH

#define MAXJSAMPLE	255


#define RGB_COLOR_COMPONENTS 3

#define MAX_COLOR_COMPONENTS 3

#define MCU_MAXLENGTH       4+1+1

//#define MAX_HUFF_TBLS       4	/* Huffman tables are numbered 0..3 */
#define MAX_HUFF_TBLS       2	/* Huffman tables are numbered 0..1 */
#define MAX_QUANT_TBLS      4

#define BITS_IN_J3D_SAMPLE  8   /* always 8 bits in j3d_sample */

typedef long int32;
typedef unsigned char int8;

typedef int j3d_sample; /* int for efficiency reasons. 1 byte would be enough, but slower */

typedef int8 *j3d_rgb_image; /* RGB raw image data, must be int8 */


typedef j3d_sample * j3d_color_component;     /* the color components */
typedef j3d_color_component * j3d_image;      /* downsampled image */

typedef unsigned int * j3d_quant_tbl;


typedef struct {
    unsigned int depth;
    unsigned int width; /* ToDo: I think this should be an array (elements for each color_component) */
    unsigned int height;
    j3d_image * images;  /* array(max_depth) of images */
} j3d_image_sequence_struct;
typedef j3d_image_sequence_struct *j3d_image_sequence_ptr;



typedef int j3d_dct_coef;     /* DCT coefficient */
typedef j3d_dct_coef * j3d_dct_coefcube;       /* a cube has size DCTSIZExDCTSIZExdepth */
typedef j3d_dct_coefcube * j3d_dct_cubearray;  /* array of DCT-coefficient cubes  */


typedef struct {
    unsigned int depth;
    unsigned int * width;            /* amount of cubes (per component) per line */
    unsigned int * length;           /* amount of cubes (per component) in array */
    j3d_dct_cubearray * color_component;  /* array of DCT-coefficient cubes  */
} j3d_dct_coef_struct;
typedef j3d_dct_coef_struct *j3d_dct_coef_ptr;




typedef struct {
    unsigned char * next_output_byte;	/* => next byte to write in buffer */
    unsigned int free_in_buffer;	/* # of byte spaces remaining in buffer */
    int32 put_buffer;		/* current bit-accumulation buffer */
    int put_bits;			/* # of bits now in it */
    unsigned int bits_in_buffer;

    unsigned int size;   /* size (bytes) of array */
    int8 * bitarray; 
} j3d_bitarray_struct;
typedef j3d_bitarray_struct *j3d_bitarray_ptr;



/* Huffman coding tables. */

typedef struct {
    /* These two fields directly represent the contents of a JPEG DHT marker */
    int8 bits[17];		/* bits[k] = # of symbols with codes of */
    /* length k bits; bits[0] is unused */
    int8 huffval[256];		/* The symbols, in order of incr code length */
} j3d_huff_tbl;


/* Linked list for filenames */
typedef struct j3d_filelist {
    char * filename;
    struct j3d_filelist * next;
} j3d_filelist_struct;
typedef j3d_filelist_struct * j3d_filelist_ptr;

    


struct j3d_compress_struct {
    j3d_filelist_ptr next_file; /* next file in filelist queue */
    int output_file;       /* filedescriptor for output image */

    unsigned int input_bytes;  /* this is used to calculate the compression ratio */
    unsigned int output_bytes;

    unsigned int width;   /* input image dimensions */
    unsigned int height;
    unsigned int max_depth;  /* depth of imagesequence for compression */
    
    unsigned int color_components;
    unsigned int downsampling_factor[MAX_COLOR_COMPONENTS]; /* downsampling factor for each color component */

    unsigned int restart_interval; /* #cubes in a GOC (group fo cubes) */


    int sequence_length;
    
    
    

    /* coef_zigzag_order_8[i] is the zigzag-order position of the i'th element
     * of a DCT cube (depth=8) read in natural order (left to right, top to bottom).
     */
    unsigned int * coef_zigzag_order_8;
    
    /* ptrs to Huffman coding tables, or NULL if not defined */
    j3d_huff_tbl * dc_huff_tbl_ptrs[MAX_HUFF_TBLS];
    j3d_huff_tbl * ac_huff_tbl_ptrs[MAX_HUFF_TBLS];
    int dc_huff_tbl_no[MAX_COLOR_COMPONENTS];
    int ac_huff_tbl_no[MAX_COLOR_COMPONENTS];
    int huff_tbl_sent[2][MAX_HUFF_TBLS]; /* TRUE if table has already been sent by DHT */


    
    j3d_quant_tbl quant_tbl_ptrs[MAX_QUANT_TBLS];
    int quant_tbl_no[MAX_COLOR_COMPONENTS];  /* quant-tbl per component */
    int quant_tbl_sent[MAX_QUANT_TBLS]; /* TRUE if table has already been sent by DQT */
    
    /*
     * Links to compression subobjects (methods and private variables of modules)
     */
    struct j3d_c_main_controller * main;
    struct j3d_c_prep_controller * prep;
    struct j3d_c_coef_controller * coef;
    struct j3d_c_input * input;
    struct j3d_c_color_converter * cconvert;
    struct j3d_c_forward_3ddct * f3ddct;
    struct j3d_c_mcu * mcu;
    struct j3d_c_vlc * vlc;
    struct j3d_c_output * output;
    struct j3d_c_marker * marker;
};
typedef struct j3d_compress_struct *j3d_compress_ptr;




struct j3d_decompress_struct {
    int input_file;       /* filedescriptor for input image */
    char* output_filename;
    
    unsigned int width;   /* input image dimensions */
    unsigned int height;
    unsigned int max_depth;  /* depth of imagesequence for compression */
    
    unsigned int color_components;
    unsigned int downsampling_factor[MAX_COLOR_COMPONENTS]; /* downsampling factor for each color component */

    unsigned int restart_interval; /* #cubes in a GOC (group fo cubes) */

    
    /* coef_zigzag_order_8[i] is the zigzag-order position of the i'th element
     * of a DCT cube read in natural order (left to right, top to bottom).
     */
    unsigned int * coef_zigzag_order_8;
    
    /* ptrs to Huffman coding tables, or NULL if not defined */
    j3d_huff_tbl * dc_huff_tbl_ptrs[MAX_HUFF_TBLS];
    j3d_huff_tbl * ac_huff_tbl_ptrs[MAX_HUFF_TBLS];
    int dc_huff_tbl_no[MAX_COLOR_COMPONENTS];
    int ac_huff_tbl_no[MAX_COLOR_COMPONENTS];


    
    /* These parameters are never carried across datastreams, since they
     * are given in SOF/SOS markers or defined to be reset by SOI.
     */

//    int data_precision;		/* bits of precision in image data */
    
    int8 unread_marker;

    j3d_quant_tbl quant_tbl_ptrs[MAX_QUANT_TBLS];
    int quant_tbl_no[MAX_COLOR_COMPONENTS];  /* quant-tbl per component */
    
    /*
     * Links to decompression subobjects (methods and private variables of modules)
     */
    struct j3d_d_main_controller * main;
    struct j3d_d_coef_controller * coef;
    struct j3d_d_input * input;
    struct j3d_d_marker * marker;
    struct j3d_d_mcu * mcu;
    struct j3d_d_vlc * vlc;
    struct j3d_d_inverse_3ddct * i3ddct;
    struct j3d_d_post_controller * postct;
    struct j3d_d_color_deconverter * cconvert;
    struct j3d_d_output * output;
};
typedef struct j3d_decompress_struct *j3d_decompress_ptr;





#define J3D_ERROR              0 /* some error ocurred */
#define J3D_REACHED_SOS        1 /* Reached start of new scan */
#define J3D_REACHED_EOI        2 /* Reached end of image */
#define J3D_ROW_COMPLETED      3 /* Completed one iMCU row */



/* These marker codes are exported since applications and data source modules
 * are likely to want to use them.
 */

#define J3D_RST0	0xD0	/* RST0 marker code */
#define J3D_EOI	        0xD9	/* EOI marker code */
#define J3D_APP0	0xE0	/* APP0 marker code */
#define J3D_COM	        0xFE	/* COM marker code */






extern void j3d_c_set_std_params();
extern void j3d_set_quality();

extern int j3d_c_add_input_file();

extern j3d_compress_ptr j3d_create_compress();
extern void j3d_init_compress();
extern void j3d_start_compress();

extern float j3d_get_compress_ratio();


extern void j3d_d_set_std_params();

extern j3d_decompress_ptr j3d_create_decompress();
extern void j3d_init_decompress();
extern void j3d_start_decompress();


/*
 * The JPEG library modules define JPEG_INTERNALS before including this file.
 * The internal structure declarations are read only when that is true.
 * Applications using the library should not include jpegint.h
 */
#ifdef J3D_INTERNALS
#include "j3dint.h"		/* fetch private declarations */
#endif


#endif  /* J3DLIB_H */

